#ifndef STRIPEDHORSE_GREENSQUAREDETECTOR_GREENSQUAREDETECTOR_H
#define STRIPEDHORSE_GREENSQUAREDETECTOR_GREENSQUAREDETECTOR_H

#include <vector>
#include <lemonpdf/gfxstandardmanagedcontext.h>
#include "greensquare.h"

/**
 * This class is a custom implementation of a PDF graphics context. This context
 * subclases from the StandardManagedContext (which handles common operations such
 * as matrix manipulation), and extends it just enough to be able to detect Green
 * Squares (see @ref GreenSquare), which it does by intercepting path-drawing and
 * text-showing operations.
 */
class GreenSquareDetector : public LemonPDF::Graphics::StandardManagedContext
{
public:
	void closePath() override;
	const std::vector<GreenSquare> &getGreenSquares() const;
	void intersectClippingPath(enum LemonPDF::Graphics::FillType fillType) override;
	void setTextFont(LemonPDF::Graphics::FontPtr font, double size) override;
	void showText(const LemonPDF::Graphics::GlyphData &glyphData) override;
	void showText(const std::vector<LemonPDF::Graphics::SpacedText> &text) override;
	void fill(enum LemonPDF::Graphics::FillType) override;
	void fillAndStroke(enum LemonPDF::Graphics::FillType) override;
	void stroke() override;

private:
	void checkTopPath();
	std::vector<GreenSquare> m_greenSquares;
	LemonPDF::Graphics::FontPtr m_font;
};

#endif

