#include <sstream>
#include "stringutils.h"

bool StringUtilities::TryGetNextToken(std::istream &in, std::string &token, char delim)
{
	if (std::getline(in, token, delim))
		return true;

	// uh oh
	in.setstate(std::ios::failbit);
	return false;
}

bool StringUtilities::TryParseMeasurement(const std::string &input, Measurement &m)
{
	std::istringstream inputStream(input);
	if (!(inputStream >> m.value))
		return false;
	
	char leftover;
	if (inputStream >> leftover)
	{
		if (leftover == '%')
		{
			m.unit = Measurement::Unit::kPercent;
			return true;
		}

		return false;
	}

	m.unit = Measurement::Unit::kUserUnits;
	return true;
}

bool StringUtilities::TryParseRectangle(std::istream &in, LemonPDF::Rectangle &rect, Measurement::Unit &unit)
{
	// get rectangle measurements
	Measurement measurements[4];
	for (int i = 0; i < 4; i++)
	{
		std::string measurement;
		if (!StringUtilities::TryGetNextToken(in, measurement))
			return false;
		
		if (!StringUtilities::TryParseMeasurement(measurement, measurements[i]))
		{
			in.setstate(std::ios::failbit);
			return false;
		}

		// maybe allow this at one point, but not now
		if (i > 1 && measurements[i].unit != measurements[0].unit)
			throw std::domain_error("all measurements must have the same unit");
	}

	rect = LemonPDF::Rectangle(measurements[0].value, measurements[1].value, measurements[2].value, measurements[3].value);
	unit = measurements[0].unit;

	return true;
}
