#ifndef STRIPEDHORSE_GREENSQUARE_STRINGUTILS_H
#define STRIPEDHORSE_GREENSQUARE_STRINGUTILS_H

#include <string>
#include <iostream>
#include <lemonpdf/pdftypes.h>

#include "measurement.h"

namespace StringUtilities
{
	/**
	 * @function tryGetNextToken
	 * @param in the input stream that contains tokens to extract
	 * @param[out] token an output parameter that receives the extracted token (if successful)
	 * @param delim the delimiter to use when extracting tokens
	 * @description This function can be used to parse a comma-delimited list (unless the @a delim
	 * argument is changed. If a token was successfully extracted, then this function returns true,
	 * otherwise, this function will set the fail bit in the input stream and return false.
	 */
	bool TryGetNextToken(std::istream &in, std::string &token, char delim = ',');

	/**
	 * @function tryParseMeasurement
	 * @param input a string containing a measurement to parse
	 * @param[out] m the measurement that was parsed
	 * @description Use this function to parse a measurement. For example, this function
	 * should successfully parse "1.23" or "1.23%". After parsing the measurement, it will then look
	 * at the next character. If there is no subsequent character, this function shall return true and the
	 * @a mu argument is set to kUserUnits. If there is a subsequent character, and it is a percent sign,
	 * then this function shall return true and set @ru to kPercent. If there is a subsequent character
	 * but it is not a percent symbol, then this function shall return false.
	 */
	bool TryParseMeasurement(const std::string &input, Measurement &m);

	/**
	 * @function TryParseRectangle
	 * @param in the input stream to parse
	 * @param[out] rect the resulting rectangle, if parsed successfully
	 * @param[out] mu the unit with which all values in the rectangle are represented in
	 * @description This function will attempt to parse 4 comma-delimited values and return
	 * a rectangle bound by those four values (left, bottom, width, height). If each of the
	 * values that was parsed was suffixed with a percent symbol, then the @a mu argument
	 * will be set to kPercent, otherwise, it will be set with kUserUnit.
	 */
	bool TryParseRectangle(std::istream &in, LemonPDF::Rectangle &rect, Measurement::Unit &mu);
}

#endif
