#include <regex>
#include <sstream>
#include "templater.h"

Templater::Templater(const Dictionary &d)
: m_dictionary(d)
{
	// allow a double-dollar to be replaced with a single dollar
	m_dictionary.emplace("$", "$");
}

std::string Templater::SubstituteText(const std::string &in, const Templater::Dictionary &dict)
{
	Templater t(dict);
	return t.substitute(in);
}

std::string Templater::substitute(const std::string &tmpl) const
{
	static const std::regex escape("\\$([A-Za-z0-9]+|\\$)");
	std::ostringstream result;
	
	std::string::size_type lastReplacement = 0;
	
	std::sregex_iterator templateStart(tmpl.begin(), tmpl.end(), escape);
	std::sregex_iterator templateEnd;
	
	for (std::sregex_iterator iter = templateStart; iter != templateEnd; iter++)
	{
		const std::string &word = iter->str(1);
		auto definition = m_dictionary.find(word);
		if (definition == m_dictionary.end())
			continue;
		
		result << tmpl.substr(lastReplacement, iter->position() - lastReplacement);
		result << definition->second;
		
		lastReplacement = iter->position() + iter->length();
	}
	
	result << tmpl.substr(lastReplacement);
	
	return result.str();		
}
