#ifndef STRIPEDHORSE_GREENSQUARE_TEMPLATER_H
#define STRIPEDHORSE_GREENSQUARE_TEMPLATER_H

#include <string>
#include <unordered_map>

/**
 * @class Templater
 * @description A basic class that can perform simple substitution of
 * variables within a string. Variables are alphanumerically named and
 * prefixed with a dollar symbol, e.g. $var1, $name, etc. If an actual
 * dollar symbol is required, then two dollar signs should be used in the
 * input, e.g. "$$4.00".
 */
class Templater
{
public:
	using Dictionary = std::unordered_map<std::string, std::string>;

	/**
	 * @function Templater
	 * @param dict the dictionary that maps variable names to the values
	 * that they should be replaced with. The names in this dictionary
	 * should not be prefixed with the $ symbol.
	 */
	Templater(const Dictionary &dict);

	/**
	 * @function SubstituteText
	 * @param in the input text containing variables to replace
	 * @param dict the dictionary mapping variable names to values
	 * @description Use this static function to perform one-off substitutions.
	 * For very many lots of substitutions, it is slightly more efficient to
	 * construct an instance (to prevent needless copying of the dictionary).
	 */
	static std::string SubstituteText(const std::string &in, const Dictionary &dict);
	
	/**
	 * @function substitute
	 * @param in the input text containing variables to replace
	 * @description This function will substitute any occurrences of
	 * variables in the string with the associated values in the
	 * dictionary that was supplied during construction.
	 */
	std::string substitute(const std::string &in) const;


private:
	Dictionary m_dictionary;
};

#endif
