#include "stringutils.h"
#include "textobject.h"

TextObject::TextObject()
{
}

TextObject::TextObject(Measurement::Unit mu, const LemonPDF::Rectangle &rect, const std::string &text)
: m_rectangleUnit(mu)
, m_rectangle(rect)
, m_text(text)
{
}

const LemonPDF::Rectangle &TextObject::getRectangle() const
{
	return m_rectangle;
}

Measurement::Unit TextObject::getRectangleUnit() const
{
	return m_rectangleUnit;
}

const std::string &TextObject::getText() const
{
	return m_text;
}

void TextObject::setRectangle(const LemonPDF::Rectangle &rect)
{
	m_rectangle = rect;
}

void TextObject::setRectangleUnit(Measurement::Unit mu)
{
	m_rectangleUnit = mu;
}

void TextObject::setText(const std::string &text)
{
	m_text = text;
}

std::istream &operator>>(std::istream &in, TextObject &t)
{
	std::string typeName;
	if (!StringUtilities::TryGetNextToken(in, typeName))
		return in;
	
	if (typeName != "Text")
	{
		in.setstate(std::ios_base::failbit);
		return in;
	}

	LemonPDF::Rectangle rect;
	Measurement::Unit unit;

	if (!StringUtilities::TryParseRectangle(in, rect, unit))
	{
		in.setstate(std::ios_base::failbit);
		return in;
	}

	t.setRectangle(rect);
	t.setRectangleUnit(unit);

	std::string remainder;
	std::getline(in, remainder);
	t.setText(remainder);

	return in;
}
