#ifndef STRIPEDHORSE_GREENSQUARE_TEXTOBJECT_H
#define STRIPEDHORSE_GREENSQUARE_TEXTOBJECT_H

#include <iostream>
#include <string>
#include <lemonpdf/pdftypes.h>

#include "measurement.h"

/**
 * @class TextObject
 * @description This class represents an area on a page that is associated
 * with some text. The rectangle represents the "bounding box" of the text.
 */
class TextObject
{
public:
	TextObject();

	/**
	 * @function TextObject
	 * @param mu the unit that the values in the rectangle are represented in
	 * @param rect the "bounding box" of the text object (e.g. the area that
	 * the text should occupy)
	 * @param text the text to place inside the bounding box
	 */
	TextObject(Measurement::Unit mu, const LemonPDF::Rectangle &rect, const std::string &text);

	const std::string &getFont() const;
	const LemonPDF::Rectangle &getRectangle() const;
	Measurement::Unit getRectangleUnit() const;
	const std::string &getText() const;
	void setFont(const std::string &font);
	void setRectangle(const LemonPDF::Rectangle &r);
	void setRectangleUnit(Measurement::Unit mu);
	void setText(const std::string &s);

private:
	Measurement::Unit m_rectangleUnit;
	LemonPDF::Rectangle m_rectangle;
	std::string m_font;
	std::string m_text;
};

std::istream &operator>>(std::istream &in, TextObject &t);

#endif
