#ifndef LEMONGFX_BEZIER_H
#define LEMONGFX_BEZIER_H

#include <vector>
#include "gfxtypes.h"

namespace LemonGFX
{
	namespace Bezier
	{
		/**
		Calculates a point somewhere in the middle of two points
		@param[in] p1 the first point
		@param[in] p2 the seccond point
		@param[in] t the bias (0.5 is exact midpoint). The "t" term is
			borrowed from Bezier curve approximation terminology
		@return A point representing the calculated midpoint
		*/
		PointF MidPoint(const PointF &p1, const PointF &p2, double t = 0.5);

		/**
		Approximates a quadratic Bezier curve
		@param[in] p1 the start point
		@param[in] cp the control point
		@param[in] p2 the end point
		@return A vector of points that approximate the curve
		*/
		std::vector<PointF> ApproximateCurve(const PointF &p1, const PointF &cp, const PointF &p2);

		/**
		Approximates a cubic Bezier curve
		@param[in] p1 the start point
		@param[in] cp1 the first control point
		@param[in] cp2 the second control point
		@param[in] p2 the end point
		@return A vector of points that approximate the curve
		*/
		std::vector<PointF> ApproximateCurve(const PointF &p1, const PointF &cp1, const PointF &cp2, const PointF &p2);
	}
}

#endif
