#ifndef LEMONGFX_DRAW_H
#define LEMONGFX_DRAW_H

#include <vector>
#include "bitmap.h"
#include "filltypes.h"
#include "path.h"

namespace LemonGFX
{
	namespace Draw
	{
		/**
		Does a fill and/or stroke of the provided path
		@param[in] path the path to draw (assumed to be in 10dpmm scale)
		@param[in] dpmm the number of dots per millimetre for the bitmap (12.0 == 305dpi)
		@return A bitmap containing the filled/stroked path
		*/
		Bitmap DrawPath(const Path &path, double dpmm);

		/**
		Does a fill and/or stroke of the provided paths
		@param[in] paths the paths to draw (assumed to be in 10dpmm scale)
		@param[in] dpmm the number of dots per millimetre for the bitmap (12.0 == 305dpi)
		@return A bitmap containing the filled/stroked paths
		*/
		Bitmap DrawPaths(const std::vector<Path> &paths, double dpmm);

		/**
		Does a fill and/or stroke of the provided paths on the supplied bitmap
		@param[in] bitmap the bitmap to draw onto, with respect to the bitmaps origin
		@param[in] paths the paths to draw (assumed to be in 10dpmm scale)
		@param[in] dpmm the number of dots per millimetre for the bitmap (12.0 == 305dpi)
		*/
		void DrawPathsOnBitmap(Bitmap &bitmap, const std::vector<Path> &paths, double dpmm);

		/**
		Determines whether a bit should be enabled depending on the
		coordinate and given luminance (e.g. intensity)
		*/
		bool ShouldEnableBitForCoordinate(size_t x, size_t y, uint8_t luminance);
	}
}

#endif
