#ifndef LEMONGFX_HALFTONER_H
#define LEMONGFX_HALFTONER_H

#include <array>
#include "exception.h"
#include "gfxtypes.h"

namespace LemonGFX
{
	class HalftoningException : public Exception
	{
	public:
		HalftoningException(const std::string &msg) : Exception(msg) {}
	};

	enum class HalftoningCurve
	{
		kLightest,
		kLighter,
		kLight,
		kNormal,
		kDark,
		kDarker,
		kDarkest
	};

	class SimpleHalftoner
	{
	public:
		SimpleHalftoner(HalftoningCurve hc);

		inline bool isEnabledForPoint(size_t x, size_t y, unsigned char luminance) const
		{
			return m_halftones[m_luminanceLookup[luminance]][y % 4][x % 4];
		}

		inline bool isPure(unsigned char luminance) const
		{
			return m_luminanceLookup[luminance] == 8 || m_luminanceLookup[luminance] == 0;
		}

		inline uint16_t getBitPatternForLuminance(unsigned char luminance) const
		{
			return m_bitPatterns[m_luminanceLookup[luminance]];
		}
	
	private:
		std::array<uint16_t, 9> m_bitPatterns;
		bool m_halftones[9][4][4];
		std::array<unsigned char, 256> m_luminanceLookup;
	};

	/**
	 * Returns a HalftoningCurve enum value based on a string name. If it is an invalid
	 * name, this function will return kNormal
	 * 
	 * @param[in] name the name of the halftoning curve, e.g. Lightest, or Dark. Case sensitive.
	 */
	HalftoningCurve HalftoningCurveFromName(const std::string &name);
}

#endif
