#ifndef LEMONGFX_IMAGE_H
#define LEMONGFX_IMAGE_H

#include <cstdint>
#include <iostream>
#include <vector>
#include "exception.h"
#include "gfxtypes.h"

namespace LemonGFX
{
	// A "Bitmap" is strictly a "bit map" (1 bit per pixel), whereas
	// an "Image" can represent an image containing more than one
	// bit per sample and more than one sample per pixel
	class Image
	{
	public:
		enum class Format
		{
			kTIFF
		};

		Image();
		Image(size_t bps, size_t spp, size_t w, size_t h);
		Image(size_t bps, size_t spp, size_t w, size_t h, const Buffer &);
		Image(size_t bps, size_t spp, size_t w, size_t h, Buffer &&);
		Image(const Image &);
		Image(Image &&);
		Image &operator=(const Image &);
		virtual ~Image() {}

		virtual bool blackIs1() const;
		virtual size_t getBitsPerSample() const;
		virtual size_t getBytesPerRow() const;
		virtual const Buffer &getData() const;
		virtual Buffer &getData();
		virtual size_t getDataSize() const;
		virtual size_t getHeight() const;
		virtual size_t getSamplesPerPixel() const;
		virtual size_t getXResolution() const;
		virtual size_t getYResolution() const;
		virtual size_t getWidth() const;
		virtual void invert();
		virtual bool isValidSize() const;
		virtual void read(std::istream &);
		virtual void reserve(size_t s);
		virtual void setResolution(size_t x, size_t y);
		virtual void write(Format, std::ostream &) const;

	protected:
		size_t m_bitsPerSample;
		size_t m_samplesPerPixel;
		size_t m_width;
		size_t m_height;
		size_t m_xResolution;
		size_t m_yResolution;
		Buffer m_data;

		// derived but saved for efficiency
		size_t m_bytesPerRow;
	};

	class ImageException : public Exception
	{
	public:
		ImageException(const std::string &msg) : Exception(msg) {}
	};
}

#endif
