#ifndef LEMONGFX_PATH_H
#define LEMONGFX_PATH_H

#include <vector>
#include "filltypes.h"
#include "gfxtypes.h"

namespace LemonGFX
{
	class Path
	{
	public:
		typedef std::vector<PointF> Subpath;
		typedef std::vector<Subpath> SubpathCollection;

		Path();
		Path(const SubpathCollection &m_subpaths);
		Path(const Path &);
		Path(Path &&);

		Subpath &operator[](SubpathCollection::size_type i);
		const Subpath &operator[](SubpathCollection::size_type i) const;

		Path &operator=(const Path &other);

		void addSubpath(const Subpath &subpath);
		Subpath &back();
		const Subpath &back() const;
		SubpathCollection::const_iterator begin() const;
		void clearSubpaths();
		void emplaceSubpath(Subpath &&subpath);
		SubpathCollection::const_iterator end() const;
		Subpath &front();
		const Subpath &front() const;
		RectangleF getBounds() const;
		double getFillShade() const;
		enum Fill::FillType getFillType() const;
		SubpathCollection::size_type getSize() const;
		double getStrokeShade() const;
		double getStrokeWidth() const;
		bool isDesiredShapeForTemplating() const;
		bool isFilled() const;
		void setFillShade(double fillShade);
		void setFillType(enum Fill::FillType ft);
		void setIsFilled(bool isFilled);
		void setStrokeShade(double strokeShade);
		void setStrokeWidth(double strokeWidth);

	private:
		SubpathCollection m_subpaths;
		bool m_isFilled;
		double m_fillShade;
		Fill::FillType m_fillType;
		double m_strokeShade;
		double m_strokeWidth;
	};
}

#endif
