#ifndef LEMONGFX_REUSS_HALFTONER_H
#define LEMONGFX_REUSS_HALFTONER_H

#include <array>
#include "gfxtypes.h"
#include "halftoner.h"

namespace LemonGFX
{
	class ReussHalftoner200dpi
	{
	public:
		ReussHalftoner200dpi(HalftoningCurve hc)
		: m_curve(hc), m_halftones {
#include "reuss-200dpi-halftones-modified.txt"
		}
		{
			if (m_curve != HalftoningCurve::kNormal)
				throw HalftoningException("Only normal halftoning curve supported by this halftoner");
		}

		inline bool isEnabledForPoint(size_t x, size_t y, unsigned char luminance) const
		{
			return !m_halftones[luminance][(y % 16) * 16 + x % 16];
		}

		inline bool isPure(unsigned char luminance) const
		{
			return luminance == 0 || luminance == 255;
		}

	private:
		HalftoningCurve m_curve;
		bool m_halftones[256][256];
	};

	class ReussHalftoner300dpi
	{
	public:
		ReussHalftoner300dpi(HalftoningCurve hc)
		: m_curve(hc)
		, m_halftones {
#include "reuss-300dpi-halftones-modified.txt"
		}
		{
			if (m_curve != HalftoningCurve::kNormal)
				throw HalftoningException("Only normal halftoning curve supported by this halftoner");
		}

		inline bool isEnabledForPoint(size_t x, size_t y, unsigned char luminance) const
		{
			return !m_halftones[luminance][(y % 16) * 16 + x % 16];
		}

		inline bool isPure(unsigned char luminance) const
		{
			return luminance == 0 || luminance == 255;
		}

	private:
		HalftoningCurve m_curve;
		bool m_halftones[256][256];
	};

	class ReussHalftoner600dpi
	{
	public:
		ReussHalftoner600dpi(HalftoningCurve hc)
		: m_curve(hc)
		, m_halftones {
#include "reuss-600dpi-halftones-modified.txt"
		}
		{
			if (m_curve != HalftoningCurve::kNormal)
				throw HalftoningException("Only normal halftoning curve supported by this halftoner");
		}

		inline bool isEnabledForPoint(size_t x, size_t y, unsigned char luminance) const
		{
			return !m_halftones[luminance][(y % 32) * 32 + x % 32];
		}

		inline bool isPure(unsigned char luminance) const
		{
			return luminance == 0 || luminance == 255;
		}

	private:
		HalftoningCurve m_curve;
		bool m_halftones[256][1024];
	};
}

#endif