#ifndef LEMONPDF_CMAP_H
#define LEMONPDF_CMAP_H

#include <iostream>
#include <map>

namespace LemonPDF
{
	class CMap
	{
	public:
		CMap();
		CMap(const CMap &other);
		CMap(CMap &&other);
		
		static CMap FromPostscriptStyle(std::istream &in);
		static CMap MacRomanEncoding();
		static CMap WinAnsiEncoding();
		static std::string ToUTF8(char16_t);
		static std::string ToUTF8(char32_t);
		
		CMap &operator=(const CMap &other);

		void addMapping(char32_t, const std::string &);
		std::string getCharactersForGlyphIndex(char32_t index) const;
		std::string getCharactersFromGlyphIndexString(const std::u16string &input, const std::string &missing = "?") const;
		std::string getCharactersFromGlyphIndexString(const std::string &input, const std::string &missing = "?") const;
		char32_t getGlyphIndexForString(const std::string &str) const;
		bool hasMappingForGlyphIndex(char32_t index) const;
		bool hasMappings() const;
		bool isKnownMapping() const;
		bool tryGetGlyphIndexForString(const std::string &str, char32_t &result) const;

	private:
		CMap(char16_t maxCode, char16_t offset, const std::vector<char16_t> &mapping);
		std::map<char32_t, std::string> m_forwardMapping;
		std::map<std::string, char32_t> m_reverseMapping;
		bool m_isKnownMapping;
	};
}

#endif
