#ifndef LEMONPDF_GFXCOLOR_H
#define LEMONPDF_GFXCOLOR_H

#include <memory>
#include <string>
#include <vector>

#include "gfxcolorspace.h"
#include "pdfexception.h"

namespace LemonPDF
{
	namespace Graphics
	{
		class Color;
		typedef std::shared_ptr<Color> ColorPtr;
		
		class Color
		{
		public:
			Color(ColorSpacePtr cs, const std::vector<double> &comps);
			Color(ColorSpacePtr cs, const std::initializer_list<double> &comps);
			virtual ~Color();
			
			ColorSpacePtr getColorSpace() const;
			const std::vector<double> &getComponents() const;
			std::tuple<double, double, double> getHueSaturationValue() const;
			std::tuple<double, double, double> getRedGreenBlue() const;
			std::string toRGBString() const;
			
		private:
			ColorSpacePtr m_colorSpace;
			std::vector<double> m_components;
		};
		
		class ColorException : public PDFException
		{
		public:
			explicit ColorException(const std::string &msg);
		};
	}
}

#endif
