#ifndef LEMONPDF_GFXCOLORSPACE_H
#define LEMONPDF_GFXCOLORSPACE_H

#include <memory>
#include <string>
#include <vector>

#include "gfxcolorspace.h"
#include "pdfexception.h"

namespace LemonPDF
{
	namespace Graphics
	{
		class ColorSpace;
		typedef std::shared_ptr<ColorSpace> ColorSpacePtr;
		
		class XYZPoint
		{
		public:
			XYZPoint(double x, double y, double z);
			double getX() const;
			double getY() const;
			double getZ() const;
			
		private:
			double m_x;
			double m_y;
			double m_z;
		};
		
		class ColorSpace
		{
		public:
			enum class ColorModel
			{
				kGray,
				kRGB,
				kCMYK,
				kHSV,
				kCIEXYZ,
				kCIELAB,
				kOther
			};

			ColorSpace(const std::string &name);
			virtual ~ColorSpace();
			
			static ColorSpacePtr GetNamedColorSpace(const std::string &name);
			static ColorSpacePtr CreateICCBasedColorSpace(const std::vector<unsigned char> &data, size_t numberOfComponents);
			static ColorSpacePtr CreateCalibratedGrayColorSpace(const XYZPoint &white, const XYZPoint &black, double gg);
			static ColorSpacePtr CreateCalibratedRGBColorSpace(const XYZPoint &white, const XYZPoint &black, double rg, double gg, double bg);
			
			virtual ColorModel getColorModel() const = 0;
			virtual const std::string &getName() const;
			virtual size_t getNumberOfComponents() const = 0;
			virtual bool isSupported() const = 0;
		protected:
			ColorSpace();
		
		private:
			std::string m_name;
		};
	}
}

#endif
