#ifndef LEMONPDF_PATH_H
#define LEMONPDF_PATH_H

#include <vector>
#include "pdftypes.h"

namespace LemonPDF
{
	namespace Graphics
	{
		class Path
		{
		public:
			enum class PathNodeType
			{
				kNewPath,
				kLine,
				kBezierCurve,
				kClosePath,
				kTerminal
			};
			
			class PathNode
			{
			public:
				template<typename... Args>
				PathNode(enum PathNodeType pathNodeType, Args... args)
				  : m_pathNodeType(pathNodeType), m_points({args...})
				{
				}
				
				bool operator==(const PathNode &other) const;

				enum PathNodeType getPathNodeType() const;
				std::vector<LemonPDF::Point> &getPoints();
				const std::vector<LemonPDF::Point> &getPoints() const;
				void setPathNodeType(enum PathNodeType);
				
			private:
				enum PathNodeType m_pathNodeType;
				std::vector<LemonPDF::Point> m_points;
			};
			
			Path();

			static Path Transform(const Path &p, const TransformMatrix &ctm);

			bool operator==(const Path &other) const;
			
			void appendLine(const LemonPDF::Point &p);
			void appendBezierCurve(const LemonPDF::Point &c, const LemonPDF::Point &p);
			void appendBezierCurve(const LemonPDF::Point &c1, const LemonPDF::Point &c2, const LemonPDF::Point &p);
			std::vector<PathNode>::iterator begin();
			std::vector<PathNode>::const_iterator cbegin() const;
			std::vector<PathNode>::const_iterator cend() const;
			void clear();
			void closePath();
			void combine(const Path &other);
			std::vector<PathNode>::iterator end();
			void endPath();
			const std::vector<PathNode> &getPathNodes() const;
			bool isDrawable() const;
			void move(const LemonPDF::Point &p);
			bool simplify();
			std::vector<PathNode>::size_type size() const;
		
		private:
			std::vector<PathNode> m_pathNodes;
			std::vector<PathNode>::size_type m_lastMove;
		};
	}
}

#endif

