#ifndef LEMONPDF_STANDARDGRAPHICSCONTEXT_H
#define LEMONPDF_STANDARDGRAPHICSCONTEXT_H

#include "gfxcontext.h"

namespace LemonPDF
{
	namespace Graphics
	{
		// This class implements standard behaviour for managing
		// state (ctm, current point, current font etc)
		class StandardManagedContext : public IContext
		{
		public:
			StandardManagedContext();
			void concat(const TransformMatrix &) override;
			void restoreState() override;
			void saveState() override;
			void setFlatness(double) override;
			void setLineCapStyle(enum LineCapStyle) override;
			void setLineJoinStyle(enum LineJoinStyle) override;
			void setLineWidth(double) override;
			void setMiterLimit(double) override;
			void setRenderingIntent(enum RenderingIntent) override;

			// colors
			void setDefaultColorSpace(ColorSpacePtr) override;
			void setFillAlpha(double) override;
			void setFillColorComponents(const std::vector<double> &) override;
			void setFillPattern(PatternPtr) override;
			void setFillColorSpace(ColorSpacePtr) override;
			void setStrokeAlpha(double) override;
			void setStrokeColorComponents(const std::vector<double> &) override;
			void setStrokePattern(PatternPtr) override;
			void setStrokeColorSpace(ColorSpacePtr) override;

			// fonts
			void setTextCharacterSpacing(double) override;
			void setTextWordSpacing(double) override;
			void setTextScale(double) override;
			void setTextLeading(double) override;
			void setTextFont(FontPtr, double) override;
			void setTextRenderingMode(enum TextRenderingMode) override;
			void setTextRise(double) override;

			// text
			void beginText() override;
			void endText() override;
			void moveToNextLine() override;
			void moveToNextLine(const Point &) override;
			void setTextMatrix(const TransformMatrix &) override;
			
			// paths
			void closePath() override;
			void curveFrom(const Point &, const Point &) override;
			void curveTo(const Point &, const Point &) override;
			void curveTo(const Point &, const Point &, const Point &) override;
			void endPath() override;
			void lineTo(const Point &) override;
			void moveTo(const Point &) override;
			void rectangle(const Rectangle &) override;

			State &topState();

		private:
			std::vector<State> m_states;
		};
	}
}

#endif

