#include "gfxparser.h"
#include "gfxtilingpattern.h"

LemonPDF::Graphics::TilingPattern::TilingPattern(ResourceManagerPtr resources, const GFXTokenStore &tokenStore)
  : m_resources(resources), m_tokenStore(tokenStore)
{
}

const LemonPDF::Rectangle &LemonPDF::Graphics::TilingPattern::getBoundingBox() const
{
	return m_bbox;
}

const LemonPDF::TransformMatrix &LemonPDF::Graphics::TilingPattern::getMatrix() const
{
	return m_matrix;
}

const LemonPDF::Point &LemonPDF::Graphics::TilingPattern::getStep() const
{
	return m_step;
}

void LemonPDF::Graphics::TilingPattern::paint(IContext &context)
{
	m_tokenStore.reset();
	GFXParser parser(m_tokenStore, context);
	context.saveState();
	context.concat(m_matrix);
	parser.parseAll(m_resources);
	context.restoreState();
}

void LemonPDF::Graphics::TilingPattern::setBoundingBox(const Rectangle &bbox)
{
	m_bbox = bbox;
}

void LemonPDF::Graphics::TilingPattern::setMatrix(const TransformMatrix &matrix)
{
	m_matrix = matrix;
}

void LemonPDF::Graphics::TilingPattern::setStep(const Point &step)
{
	m_step = step;
}
