#ifndef LEMONPDF_GFXTOKENISER_H
#define LEMONPDF_GFXTOKENISER_H

#include <cstdint>
#include <deque>
#include <string>
#include <vector>

#include "pdfexception.h"

namespace LemonPDF
{
	namespace Graphics
	{
		class GFXToken
		{
		public:
			enum TokenType
			{
				kArrayStart,
				kArrayEnd,
				kDictStart,
				kDictEnd,
				kName,
				kNumber,
				kText,
				kOperator,
				kEndOfFile
			};
			
			GFXToken(size_t, enum TokenType, const std::string & = std::string());
			
			size_t getOffset() const;
			enum TokenType getType() const;
			std::string getNameValue() const;
			const std::string &getText() const;
		
		private:
			size_t m_offset;
			enum TokenType m_type;
			std::string m_text;
		
		friend class GFXTokeniser;
		};

		class IGFXTokeniser
		{
		public:
			virtual void consume() = 0;
			virtual GFXToken getNextToken() = 0;
			virtual GFXToken peekNextToken() = 0;

			virtual ~IGFXTokeniser() {}
		};
		
		class GFXTokeniser : public IGFXTokeniser
		{
		public:
			GFXTokeniser(std::istream &);
			
			void consume() override;
			GFXToken getNextToken() override;
			GFXToken peekNextToken() override;

		private:
			std::istream &m_stream;
			std::deque<GFXToken> m_tokens;
		};
		
		class GFXTokeniserException : public PDFException
		{
		public:
			explicit GFXTokeniserException(const std::string &msg, std::istream &);
		};
		
		class GFXTokenException : public PDFException
		{
		public:
			explicit GFXTokenException(const std::string &msg);
		};
	}
}

#endif
