#include "gfxtokenstore.h"

LemonPDF::Graphics::GFXTokenStore::GFXTokenStore(std::istream &in)
  : m_currentToken(0)
{
	bool endOfFile = false;
	GFXTokeniser normalTokeniser(in);
	
	do
	{
		GFXToken token = normalTokeniser.getNextToken();
		endOfFile = (token.getType() == GFXToken::TokenType::kEndOfFile);
		m_tokens.emplace_back(std::move(token));
	}
	while (!endOfFile);
}

LemonPDF::Graphics::GFXTokenStore::GFXTokenStore(const GFXTokenStore &other)
  : m_tokens(other.m_tokens), m_currentToken(other.m_currentToken)
{
}

LemonPDF::Graphics::GFXTokenStore::GFXTokenStore(GFXTokenStore &&other)
  : m_tokens(std::move(other.m_tokens)), m_currentToken(other.m_currentToken)
{
}

void LemonPDF::Graphics::GFXTokenStore::consume()
{
	if ((m_currentToken + 1) < m_tokens.size())
		m_currentToken++;
}

LemonPDF::Graphics::GFXToken LemonPDF::Graphics::GFXTokenStore::getNextToken()
{
	GFXToken result = peekNextToken();
	consume();
	return result;
}

LemonPDF::Graphics::GFXToken LemonPDF::Graphics::GFXTokenStore::peekNextToken()
{
	return m_tokens.at(m_currentToken);
}

void LemonPDF::Graphics::GFXTokenStore::reset()
{
	m_currentToken = 0;
}
