#ifndef LEMONPDF_GRAPHICS_TRUETYPEFONT_H
#define LEMONPDF_GRAPHICS_TRUETYPEFONT_H

#include "gfxfont.h"

namespace LemonPDF
{
	namespace Graphics
	{
		class TrueTypeFont : public Font
		{
		public:
			class Table
			{
			public:
				Table(const std::vector<unsigned char> &data);
				uint32_t getChecksum() const;
				const std::vector<unsigned char> &getData() const;
				void setData(const std::vector<unsigned char> &data);
				
			private:
				uint32_t m_checksum;
				std::vector<unsigned char> m_data;
			};

			struct DateTime
			{
				uint32_t high;
				uint32_t low;
			};

			struct FontHeader
			{
				uint16_t majorVersion;
				uint16_t minorVersion;
				uint32_t fontRevision;
				uint32_t checkSumAdjustment;
				uint32_t magicNumber;
				uint16_t flags;
				uint16_t unitsPerEm;
				DateTime created;
				DateTime modified;
				int16_t xMin;
				int16_t yMin;
				int16_t xMax;
				int16_t yMax;
				uint16_t macStyle;
				uint16_t lowestRecPPEM;
				int16_t fontDirectionHint;
				int16_t indexToLocFormat;
				int16_t glyphDataFormat;
			};

			TrueTypeFont(const std::string &name, enum FontCompleteness fc, enum FontType ft, int fw, const std::vector<unsigned char> &);
			virtual ~TrueTypeFont();

			void enableEmbeddability();
			const Table &getTable(const std::string &name) const;
			const GlyphOutlineMap &getGlyphOutlineMap() const override;
			bool hasTable(const std::string &name) const;
			void setTable(const std::string &name, const Table &table);
			void write(std::ostream &out) const;
			
		private:
			void parseCMap();
			void parseGlyf();

			std::map<std::string, Table> m_tables;
			std::map<char32_t, char32_t> m_glyphMapping;
			FontHeader m_fontHeader;

			// lazy load (parse the glyph outlines only if they're requested)
			mutable GlyphOutlineMap m_glyphOutlines;
			mutable bool m_glyphOutlinesLoaded;
		};
	}
}

#endif
