#include <limits>
#include "gfxtruetypefonttablereader.h"

namespace
{
	[[noreturn]] void noMoreData()
	{
		throw LemonPDF::PDFIOException("Cannot read past end of table");
	}
}

LemonPDF::Graphics::TrueTypeFontTableReader::TrueTypeFontTableReader(const BufferType &table, BufferType::size_type initialPosition)
  : m_table(table), m_currentPosition(initialPosition)
{
}

void LemonPDF::Graphics::TrueTypeFontTableReader::advance(BufferType::size_type offset)
{
	if (m_currentPosition + offset >= m_table.size())
		noMoreData();
	m_currentPosition += offset;
}

LemonPDF::Graphics::TrueTypeFontTableReader::BufferType::size_type LemonPDF::Graphics::TrueTypeFontTableReader::getCurrentPosition() const
{
	return m_currentPosition;
}

const LemonPDF::Graphics::TrueTypeFontTableReader::BufferType &LemonPDF::Graphics::TrueTypeFontTableReader::getData() const
{
	return m_table;
}

int8_t LemonPDF::Graphics::TrueTypeFontTableReader::readInt8()
{
	uint8_t asUnsigned = readUInt8();
	if (asUnsigned & 0x80)
		return std::numeric_limits<int8_t>::min() + static_cast<int8_t>(asUnsigned & 0x7F);
	else
		return static_cast<int8_t>(asUnsigned);
}

int16_t LemonPDF::Graphics::TrueTypeFontTableReader::readInt16()
{
	uint16_t asUnsigned = readUInt16();
	if (asUnsigned & 0x8000)
		return std::numeric_limits<int16_t>::min() + static_cast<int16_t>(asUnsigned & 0x7FFF);
	else
		return static_cast<int16_t>(asUnsigned);
}

int32_t LemonPDF::Graphics::TrueTypeFontTableReader::readInt32()
{
	uint32_t asUnsigned = readUInt32();
	if (asUnsigned & 0x8000000UL)
		return std::numeric_limits<int32_t>::min() + static_cast<int32_t>(asUnsigned & 0x7FFFFFFF);
	else
		return static_cast<int32_t>(asUnsigned);
}

uint8_t LemonPDF::Graphics::TrueTypeFontTableReader::readUInt8()
{
	if (m_currentPosition >= m_table.size())
		noMoreData();
	
	uint8_t result = m_table[m_currentPosition];
	m_currentPosition += sizeof result;
	return result;
}

uint16_t LemonPDF::Graphics::TrueTypeFontTableReader::readUInt16()
{
	if (m_currentPosition + 1 >= m_table.size())
		noMoreData();
	
	uint16_t result = (m_table[m_currentPosition] << 8UL) | m_table[m_currentPosition + 1];
	m_currentPosition += sizeof result;
	return result;
}

uint32_t LemonPDF::Graphics::TrueTypeFontTableReader::readUInt32()
{
	if (m_currentPosition + 3 >= m_table.size())
		noMoreData();
	
	uint32_t result = (m_table[m_currentPosition + 0] << 24UL) |
	                  (m_table[m_currentPosition + 1] << 16UL) |
					  (m_table[m_currentPosition + 2] <<  8UL) |
					  (m_table[m_currentPosition + 3] <<  0UL);
	m_currentPosition += sizeof result;
	return result;
}

void LemonPDF::Graphics::TrueTypeFontTableReader::seek(BufferType::size_type position)
{
	if (position >= m_table.size())
		noMoreData();
	
	m_currentPosition = position;
}
