#include <iostream>
#include "gfxcontext.h"
#include "libtextextraction.h"

namespace
{
	class TextOnlyContext : public LemonPDF::Graphics::IContext
	{
	public:
		virtual void moveToNextLine(const LemonPDF::Point &p) override
		{
			(void) p;
			m_text.push_back('\n');
		}
		
		virtual void moveToNextLine() override
		{
			m_text.push_back('\n');
		}

		virtual void setTextFont(LemonPDF::Graphics::FontPtr font, double size) override
		{
			(void) size;
			m_font = font;
		}
		
		virtual void showText(const LemonPDF::Graphics::GlyphData &glyphData) override
		{
			if (m_font)
			{
				LemonPDF::Graphics::GlyphIndices indices = m_font->getGlyphIndicesForData(glyphData);
				m_text.append(m_font->decodeText(indices));
			}
		}
		
		virtual void showText(const std::vector<LemonPDF::Graphics::SpacedText> &text) override
		{
			for (LemonPDF::Graphics::SpacedText st : text)
			{
				if (st.spacing < -100)
					m_text.push_back(' ');
				
				showText(st.glyphData);
			}
		}
		
		const std::string &getText() const
		{
			return m_text;
		}
	
	private:
		LemonPDF::Graphics::FontPtr m_font;
		std::string m_text;
	};
}

std::string LemonPDF::TextExtraction::ExtractTextFromPage(PagePtr page)
{
	TextOnlyContext ctx;
	page->render(ctx);
	return ctx.getText();
}
