#include "pdfcrypt-arc4.h"

namespace
{
	template<typename T, size_t N>
	void initialiseToIdentity(std::array<T, N> &array)
	{
		for (size_t i = 0; i < N; i++)
			array[i] = T(i);
	}
}

LemonPDF::Cryptology::ARC4::ARC4(const std::vector<unsigned char> &secret)
  : m_i(0), m_j(0)
{
	initialiseToIdentity(m_keySchedule);
	
	unsigned char j = 0;
	for (size_t x = 0; x < m_keySchedule.size(); x++)
	{
		unsigned char i = x;
		j = j + m_keySchedule[i] + secret[i % secret.size()];
		std::swap(m_keySchedule[i], m_keySchedule[j]);
	}
}

void LemonPDF::Cryptology::ARC4::crypt(std::istream &in, std::ostream &out)
{
	char ch;
	while (in.get(ch))
	{
		unsigned char k = iterate();
		out.put(static_cast<char>(static_cast<unsigned char>(ch) ^ k));
	}
}

void LemonPDF::Cryptology::ARC4::crypt(const std::string &in, std::string &out)
{
	out.resize(in.size());

	for (size_t i = 0; i < in.size(); i++)
	{
		unsigned char byte = in[i];
		unsigned char k = iterate();
		out[i] = static_cast<char>(byte ^ k);
	}
}

void LemonPDF::Cryptology::ARC4::crypt(std::string &str)
{
	for (size_t i = 0; i < str.size(); i++)
	{
		unsigned char byte = str[i];
		unsigned char k = iterate();
		str[i] = static_cast<char>(byte ^ k);
	}
}

unsigned char LemonPDF::Cryptology::ARC4::iterate()
{
	m_i++;
	m_j += m_keySchedule[m_i];
	std::swap(m_keySchedule[m_i], m_keySchedule[m_j]);
	unsigned char kindex = m_keySchedule[m_i] + m_keySchedule[m_j];
	return m_keySchedule[kindex];
}
