#include <cctype>
#include <iomanip>
#include <sstream>
#include "pdfexception.h"

namespace
{
	void printContext(std::istream &in, const std::string &msg, std::ostream &out)
	{
		size_t originalOffset  = in.tellg();
		size_t moveTo;
		
		if (originalOffset < 20)
			moveTo = 0;
		else
			moveTo = originalOffset - 20;
		
		out << msg << " (at " << originalOffset << " / 0x" << std::hex << originalOffset << ")\n";
		
		in.seekg(moveTo);
		if (moveTo > 0)
			out << "... ";
		
		for (size_t i = 0; i < 80; i++)
		{
			int ch = in.get();
			if (ch == EOF)
				break;
			
			if (::isspace(ch))
				out.put(' ');
			else if (::isprint(ch))
				out.put(static_cast<char>(ch));
			else
				out.put('.');
			
		}
		
		if (!in.eof())
			out << " ...";
		
		in.seekg(originalOffset);
	}
}

LemonPDF::PDFParseException::PDFParseException(const std::string &msg, std::istream &in)
  : PDFException(msg)
{
	std::stringstream ss;
	printContext(in, msg, ss);
	m_msg = ss.str();
}

LemonPDF::PDFSyntaxException::PDFSyntaxException(const std::string &msg, std::istream &in)
  : PDFException(msg)
{
	std::stringstream ss;
	printContext(in, msg, ss);
	m_msg = ss.str();
}
