#ifndef LEMONPDF_PDFEXCEPTION_H
#define LEMONPDF_PDFEXCEPTION_H 1

#include <exception>
#include <string>

namespace LemonPDF
{
	class PDFException : public std::exception
	{
	public:
		explicit PDFException(const std::string &msg) : m_msg(msg) {}
		virtual const char *what() const throw() { return m_msg.c_str(); }
		virtual ~PDFException() throw() {};
		
	protected:
		std::string m_msg;
	};
	
	
	class PDFParseException : public PDFException
	{
	public:
		explicit PDFParseException(const std::string &msg) : PDFException(msg) {}
		explicit PDFParseException(const std::string &msg, std::istream &i);
	};
	
	class PDFIOException : public PDFException
	{
	public:
		explicit PDFIOException(const std::string &msg) : PDFException(msg) {}
	};
	
	class PDFSyntaxException : public PDFException
	{
	public:
		explicit PDFSyntaxException(const std::string &msg, std::istream &i);
	};
	
	class PDFInvalidArgumentException : public PDFException
	{
	public:
		explicit PDFInvalidArgumentException(const std::string &msg) : PDFException(msg) {}
	};
	
	class PDFInvalidOperationException : public PDFException
	{
	public:
		explicit PDFInvalidOperationException(const std::string &msg) : PDFException(msg) {}
	};
	
	class PDFUnimplementedException : public PDFException
	{
	public:
		explicit PDFUnimplementedException(const std::string &msg) : PDFException("Feature not implemented: " + msg) {}
	};
}

#endif

