#include "pdffilter-runlengthdecode.h"

namespace
{
	[[noreturn]] void unexpectedStreamError()
	{
		throw LemonPDF::PDFFilterException("Unexpected I/O error during RLE decoding");
	}
}

LemonPDF::RunLengthDecodeFilter::RunLengthDecodeFilter()
{
}

void LemonPDF::RunLengthDecodeFilter::decode(std::istream &in, std::ostream &out)
{
	char ch;

	while (in.get(ch))
	{
		unsigned char uch = static_cast<unsigned char>(ch);
		if (uch <= 127)
		{
			// if uch is 127 or less, it means to copy the next (uch + 1) characters literally
			for (size_t i = 0; i <= uch; i++)
			{
				if (in.get(ch))
					out.put(ch);
				else
					unexpectedStreamError();
			}
		}
		else if (uch >= 129)
		{
			// if uchis 129 or greater, it means to repeat the next byte (257 - uch) times.
			size_t repeats = 257 - uch;
			if (!in.get(ch))
				unexpectedStreamError();
			
			for (size_t i = 0; i < repeats; i++)
				out.put(ch);
		}
		else
		{
			// uch is 128, which denotes EOD
			break;
		}
	}
}

void LemonPDF::RunLengthDecodeFilter::encode(std::istream &in, std::ostream &out)
{
	(void) in;
	(void) out;
	throw PDFUnimplementedException("Cannot encode RunLengthDecode forrmat");
}

std::string LemonPDF::RunLengthDecodeFilter::getName() const
{
	return "RunLengthDecode";
}