#include <sstream>
#include "pdfexception.h"
#include "pdftokeniser.h"
#include "pdfobject.h"

std::ostream &LemonPDF::operator<<(std::ostream &stream, ObjectPtr object)
{
	return stream << object->operator std::string();
}

LemonPDF::Object::Object()
{
}

LemonPDF::Object::~Object()
{
}

LemonPDF::ObjectPtr LemonPDF::Object::Parse(Tokeniser &tokeniser, ResolverFunction resolver)
{
	bool stop = false;
	while (!stop)
	{
		Token t = tokeniser.peekNextToken();
		switch (t.getType())
		{
			case Token::kDictStart:
			{
				DictionaryPtr dict = Dictionary::Parse(tokeniser);
				if (tokeniser.peekNextToken().getType() == Token::kStreamStart)
				{
					StreamPtr stream = Stream::Parse(tokeniser, resolver, dict);
					return stream;
				}
				else
				{
					return dict;
				}
			}
			
			case Token::kArrayStart:
				return Array::Parse(tokeniser);
			
			case Token::kInteger:
			case Token::kReal:
			case Token::kNull:
			case Token::kTrue:
			case Token::kFalse:
			case Token::kName:
			case Token::kReference:
			case Token::kString:
				return Literal::Parse(tokeniser);
			
			case Token::kComment:
				tokeniser.consume();
				break;
			
			default:
				throw PDFSyntaxException("Parse error", tokeniser.getInputStream());
		}
	}
	
	return nullptr;
}

LemonPDF::Object::operator std::string() const
{
	return "<Object>";
}

LemonPDF::ObjectPtr LemonPDF::Object::copy() const
{
	throw PDFInvalidOperationException("Cannot copy objects of this type");
}

void LemonPDF::Object::write(std::ostream &out, size_t depth) const
{
	(void) out;
	(void) depth;
	throw PDFUnimplementedException("Base object class is not writable");
}
