#include "pdfobject.h"

LemonPDF::ObjectReference::ObjectReference()
  : m_objectID(0), m_generation(0)
{
}

LemonPDF::ObjectReference::ObjectReference(size_t id, size_t gen)
  : m_objectID(id), m_generation(gen)
{
}

LemonPDF::ObjectReference::ObjectReference(std::pair<size_t, size_t> pair)
  : m_objectID(pair.first), m_generation(pair.second)
{
}

LemonPDF::ObjectReference::~ObjectReference()
{
}

bool LemonPDF::ObjectReference::operator<(const ObjectReference &other) const
{
	return m_objectID < other.m_objectID || (m_objectID == other.m_objectID && m_generation < other.m_generation);
}

bool LemonPDF::ObjectReference::operator==(const ObjectReference &other) const
{
	return m_objectID == other.m_objectID && m_generation == other.m_generation;
}

size_t LemonPDF::ObjectReference::getGeneration() const
{
	return m_generation;
}

size_t LemonPDF::ObjectReference::getHash() const
{
	return m_objectID << 3 | m_generation;
}

size_t LemonPDF::ObjectReference::getObjectID() const
{
	return m_objectID;
}

size_t LemonPDF::ObjectReferenceHash::operator()(const ObjectReference &objref) const
{
	return objref.getHash();
}

LemonPDF::CompressedObjectReference::CompressedObjectReference()
  : ObjectReference(0, 0), m_index(0)
{
}

LemonPDF::CompressedObjectReference::CompressedObjectReference(size_t id, size_t index)
  : ObjectReference(id, 0), m_index(index)
{
}

LemonPDF::CompressedObjectReference::CompressedObjectReference(std::pair<size_t, size_t> pair)
  : ObjectReference(pair.first, 0), m_index(pair.second)
{
}

LemonPDF::CompressedObjectReference::~CompressedObjectReference()
{
}

bool LemonPDF::CompressedObjectReference::operator==(const CompressedObjectReference &other) const
{
	return m_objectID == other.m_objectID && m_index == other.m_index;
}

bool LemonPDF::CompressedObjectReference::operator<(const CompressedObjectReference &other) const
{
	return m_objectID < other.m_objectID || m_index < other.m_index;
}

size_t LemonPDF::CompressedObjectReference::getHash() const
{
	return m_objectID << 8 | m_index;
}

size_t LemonPDF::CompressedObjectReference::getIndex() const
{
	return m_index;
}
