#ifndef LEMONPDF_PDFPAGE_H
#define LEMONPDF_PDFPAGE_H

#include <ctime>
#include <memory>
#include <string>
#include <vector>

#include "gfxcontext.h"
#include "pdfobject.h"
#include "pdfpagetreenode.h"
#include "pdfresourcemanager.h"
#include "pdftypes.h"

namespace LemonPDF
{
	class PDFFile;
	typedef std::shared_ptr<PDFFile> PDFFilePtr;
	
	class Page;
	typedef std::shared_ptr<Page> PagePtr;
	
	class Page : public PageTreeNode, public std::enable_shared_from_this<Page>
	{
	public:
		Page(PDFFilePtr pdf, DictionaryPtr object, PageTreeNodePtr parent);
		
		virtual const Rectangle &getArtBox() const;
		virtual const Rectangle &getBleedBox() const;
		virtual const std::vector<StreamPtr> &getContentStreams() const;
		virtual const Rectangle &getCropBox() const;
		virtual time_t getLastModified() const;
		virtual const Rectangle &getMediaBox() const;
		virtual const Rectangle &getTrimBox() const;
		virtual double getUserUnit() const;
		virtual ResourceManagerPtr getResources() const;

		void setArtBox(const Rectangle &r);
		void setBleedBox(const Rectangle &r);
		void setCropBox(const Rectangle &r);
		void setMediaBox(const Rectangle &r);
		void setTrimBox(const Rectangle &r);

		void render(Graphics::IContext &ctx) const;
		
		virtual ~Page();
		
	private:
		time_t m_lastModified;
		Rectangle m_mediaBox;
		Rectangle m_cropBox;
		Rectangle m_bleedBox;
		Rectangle m_trimBox;
		Rectangle m_artBox;
		ResourceManagerPtr m_resources;
		std::vector<StreamPtr> m_contentStreams;
		int m_rotation;
		double m_userUnit;
	};
}

#endif
