#ifndef LEMONPDF_PAGETREENODE_H
#define LEMONPDF_PAGETREENODE_H

#include <string>
#include <memory>
#include <vector>

#include "pdfobject.h"

namespace LemonPDF
{
	class PDFFile;
	typedef std::shared_ptr<PDFFile> PDFFilePtr;
	
	class Page;
	typedef std::shared_ptr<Page> PagePtr;
	
	class PageTreeNode;
	typedef std::shared_ptr<PageTreeNode> PageTreeNodePtr;
	typedef std::shared_ptr<const PageTreeNode> CPageTreeNodePtr;
	
	class PageTreeNode
	{
	public:
		static PageTreeNodePtr ConstructTree(PDFFilePtr pdf, DictionaryPtr object, PageTreeNodePtr root = nullptr);
		static std::vector<PagePtr> GetPageObjects(PageTreeNodePtr root);
		
		PageTreeNode(PDFFilePtr pdf, DictionaryPtr object, PageTreeNodePtr parent);
		virtual ~PageTreeNode();
		
		ObjectPtr lookup(const std::string &name) const;
		template<typename T>
		inline std::shared_ptr<T> lookup(const std::string &name) const
		{
			return std::dynamic_pointer_cast<T>(lookup(name));
		}
		
		ObjectPtr lookup(LiteralPtr l) const;
		template<typename T>
		inline std::shared_ptr<T> lookup(LiteralPtr l) const
		{
			return std::dynamic_pointer_cast<T>(lookup(l));
		}
		
		size_t getChildCount() const;
		PageTreeNodePtr getChild(size_t index) const;
		DictionaryPtr getDictionary() const;
		PDFFilePtr getFile() const;
		PageTreeNodePtr getParent() const;
		void removeChild(size_t index);
		
	protected:
		DictionaryPtr m_dictionary;
		std::weak_ptr<PDFFile> m_file;
		std::weak_ptr<PageTreeNode> m_parent;
		std::vector<PageTreeNodePtr> m_children;
	};
}

#endif
