#ifndef LEMONPDF_PDFRESOURCEHANDLER_H
#define LEMONPDF_PDFRESOURCEHANDLER_H

#include <memory>
#include "gfxcolor.h"
#include "gfxfont.h"
#include "gfxpattern.h"
#include "pdfobject.h"

namespace LemonPDF
{
	class PDFFile;

	class ResourceManager
	{
	public:
		ResourceManager(std::shared_ptr<PDFFile> pdf, DictionaryPtr resourceDictionary);
		virtual ~ResourceManager();

		virtual void addColorSpace(const std::string &name, ObjectPtr object);

		virtual Graphics::ColorSpacePtr getColorSpace(const std::string &name) const;
		virtual DictionaryPtr getDictionary() const;
		virtual std::shared_ptr<PDFFile> getFile() const;
		virtual Graphics::FontPtr getFont(const std::string &name) const;
		virtual Graphics::PatternPtr getPattern(const std::string &name) const;
		virtual StreamPtr getXObject(const std::string &name) const;

	private:
		DictionaryPtr m_resources;
		std::weak_ptr<PDFFile> m_pdf;
		
		// loading fonts doesn't alter the logical state of the ResourceManager object
		mutable std::unordered_map<std::string, Graphics::FontPtr> m_fontCache;
	};

	using ResourceManagerPtr = std::shared_ptr<ResourceManager>;
}

#endif
