#ifndef LEMONPDF_TOKENISER_H
#define LEMONPDF_TOKENISER_H 1

#include <deque>
#include <iostream>
#include <string>
#include <vector>

namespace LemonPDF
{
	class Tokeniser;
	
	class Token
	{
	public:
		enum TokenType
		{
			kDictStart,
			kDictEnd,
			kArrayStart,
			kArrayEnd,
			kObjectStart,
			kObjectEnd,
			kStreamStart,
			kStreamEnd,
			kString,
			kName,
			kInteger,
			kReal,
			kTrue,
			kFalse,
			kNull,
			kReference,
			kComment,
			kTrailer,
			kXref,
			kStartXref,
			kEndOfFile
		};
		
		Token(size_t offset, enum TokenType, const std::string & = std::string());
		
		std::pair<size_t, size_t> getObjectIDAndGeneration() const;
		size_t getOffset() const;
		enum TokenType getType() const;
		const std::string &getText() const;
		
	private:
		size_t m_offset;
		enum TokenType m_type;
		std::string m_text;
	
	friend class Tokeniser;
	};
	
	
	class Tokeniser
	{
	public:
		Tokeniser(std::istream &);
		
		void consume();
		std::istream &getInputStream();
		Token getNextToken();
		Token peekNextToken();
		std::vector<unsigned char> read(size_t);
		
	protected:
		bool shouldTokeniseMore() const;
		std::istream &m_stream;
		std::deque<Token> m_tokens;
	};
}

#endif

