#include <iomanip>
#include <sstream>
#include "pdfexception.h"
#include "pdftypes.h"

std::string LemonPDF::Types::CreateTimestamp(time_t timestamp)
{
	std::stringstream result;
	struct tm t = tm();

#ifdef _WIN32
	gmtime_s(&t, &timestamp);
#else
	gmtime_r(&timestamp, &t);
#endif

	result << "D:";
	result << std::setfill('0');
	result << std::setw(4) << t.tm_year + 1900;
	result << std::setw(2) << t.tm_mon + 1;
	result << std::setw(2) << t.tm_mday;
	result << std::setw(2) << t.tm_hour;
	result << std::setw(2) << t.tm_min;
	result << std::setw(2) << t.tm_sec;
	result << "+00'00";

	return result.str();
}

time_t LemonPDF::Types::ParseTimestamp(const std::string &value)
{
	if (value.substr(0, 2) != "D:" || value.size() < 6)
		throw PDFInvalidOperationException("Invalid date string");

	int offset = 0;

	struct tm t = tm();

	t.tm_isdst = -1;
	t.tm_year = std::stoi(value.substr(2, 4)) - 1900;

	if (value.size() >= 8)
		t.tm_mon = std::stoi(value.substr(6, 2)) - 1;

	if (value.size() >= 10)
		t.tm_mday = std::stoi(value.substr(8, 2));

	if (value.size() >= 12)
		t.tm_hour = std::stoi(value.substr(10, 2));

	if (value.size() >= 14)
		t.tm_min = std::stoi(value.substr(12, 2));

	if (value.size() >= 16)
		t.tm_sec = std::stoi(value.substr(14, 2));

	if (value.size() >= 20 && value[16] != 'Z')
	{
		int direction = (value[16] == '-' ? -1 : 1);
		
		int hours = std::stoi(value.substr(17, 2));
		int minutes = 0;
		
		if (value.size() >= 22)
			minutes = std::stoi(value.substr(20, 2));
		
		offset = direction * (hours * 60 + minutes);
	}

	return mktime(&t) + static_cast<time_t>(offset);
}
