#include <cctype>
#include "asciihex.h"
#include "exceptions.h"

namespace
{
	bool readNextByte(std::istream &in, unsigned char &result)
	{
		size_t nybbleCount = 0;
		char nybbles[3] = {0};

		char ch;
		while (in.get(ch))
		{
			if (::isspace(ch))
				continue;
			
			if (ch == '>')
				break;

			if (::isxdigit(ch))
			{
				nybbles[nybbleCount++] = ch;
				if (nybbleCount == 2)
					break;
			}
			else
			{
				throw LemonSqueezy::DecompressionException("Unexpected token in ASCIIHex stream");
			}
		}

		if (nybbleCount == 0)
			return false;
		else if (nybbleCount == 1)
			nybbles[1] = '0'; // PDF says to assume missing nybble is 0

		char *end = nullptr;
		unsigned long val = strtoul(nybbles, &end, 16);
		if (*end == '\0')
		{
			result = static_cast<unsigned char>(val);
			return true;
		}
		
		throw LemonSqueezy::DecompressionException("Unexpected error converting hex value");
	}

	char toUpperHex(unsigned char val)
	{
		static const char digits[16] = {
			'0', '1', '2', '3', '4', '5', '6', '7',
			'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
		};

		if (val > 15)
			throw LemonSqueezy::CompressionException("Internal inconsistency error when converting value to hex");
		else
			return digits[val];
	}

	char toLowerHex(unsigned char val)
	{
		static const char digits[16] = {
			'0', '1', '2', '3', '4', '5', '6', '7',
			'8', '9', 'a', 'b', 'c', 'd', 'e', 'f'
		};

		if (val > 15)
			throw LemonSqueezy::CompressionException("Internal inconsistency error when converting value to hex");
		else
			return digits[val];
	}
}

LemonSqueezy::ASCIIHexEncoding::ASCIIHexEncoding(bool uppercase, size_t lineLimit, size_t initialCharPosition)
  : LineLengthLimitedEncoding(lineLimit, initialCharPosition), m_uppercase(uppercase)
{
}

void LemonSqueezy::ASCIIHexEncoding::read(std::istream &in, std::ostream &out)
{
	unsigned char byte;
	while (readNextByte(in, byte))
		put(static_cast<char>(byte), out);
}

void LemonSqueezy::ASCIIHexEncoding::write(std::istream &in, std::ostream &out)
{
	auto toHex = m_uppercase ? toUpperHex : toLowerHex;

	char ch;
	while (in.get(ch))
	{
		unsigned char uch = static_cast<unsigned char>(ch);
		put(toHex((uch & 0xF0) >> 4), out);
		put(toHex((uch & 0x0F) >> 0), out);
	}

	put('>', out);
}
