#ifndef LEMONSQUEEZY_BITSEQUENCE_H
#define LEMONSQUEEZY_BITSEQUENCE_H

#include <climits>
#include <cstdlib>

namespace LemonSqueezy
{
	template<typename T, typename L = size_t>
	struct BasicBitSequence
	{
		using code_type = T;
		using size_type = L;
		BasicBitSequence(code_type code = code_type(), size_type length = size_type())
		  : code(code), length(length)
		{
		}

		void clear()
		{
			code = code_type();
			length = size_type();
		}

		void reverse()
		{
			code_type result = 0;
			for (size_type i = 0; i < length; i++)
				result |= ((code & (1U << i)) >> i) << (length - i - 1U);
			code = result;
		}

		code_type code;
		size_type length;
	};

	template<typename T, typename L>
	bool operator==(const BasicBitSequence<T, L> &c1, const BasicBitSequence<T, L> &c2)
	{
		return c1.length == c2.length && c1.code == c2.code;
	}

	using BitSequence = BasicBitSequence<uint32_t, size_t>;
}

#endif
