#ifndef LEMONSQUEEZY_CCITT_H
#define LEMONSQUEEZY_CCITT_H 1

#include "icompressor.h"
#include "idecompressor.h"

namespace LemonSqueezy
{
	class CCITTCompression : public IDecompressor, public ICompressor
	{
	public:
		enum class Encoding
		{
			kGroup4_2D,
			kGroup3_1D,
			kGroup3_2D
		};

		CCITTCompression(enum Encoding encoding, int errorLines, bool encodedByteAlign, size_t columns, size_t rows, bool endOfBlock, bool blackIs1);
		virtual void read(std::istream &in, std::ostream &out) override;
		virtual void write(std::istream &in, std::ostream &out) override;
		virtual ~CCITTCompression();
	
	private:
		struct BlackRange
		{
			BlackRange(size_t index, size_t length)
			  : index(index), length(length)
			{
			}

			size_t index;
			size_t length;
		};

		class DecodingLine
		{
		public:
			DecodingLine(size_t columns);
			void decodeLineIntoBuffer(std::vector<unsigned char> &buffer) const;
			size_t getB1(const DecodingLine &referenceLine) const;
			size_t getB2(const DecodingLine &referenceLine) const;
			bool isAtEnd() const;
			bool isPenDown() const;
			void movePen(size_t p);
			size_t offsetPen(size_t p);
			void togglePen();

		protected:
			size_t m_columns;
			size_t m_a0;
			bool m_penDown;
			size_t m_penDownPosition;
			std::vector<BlackRange> m_blackRanges;
		};

		class EncodingLine
		{
		public:
			EncodingLine(int columns);
			int getA0() const;
			int getA1() const;
			int getA2(int a1) const;
			int getB1(const EncodingLine &referenceLine) const;
			int getB2(const EncodingLine &referenceLine, int b1) const;

			inline std::vector<unsigned char> &getData()
			{
				return m_data;
			}

			inline bool isAtEnd() const
			{
				return m_a0 == m_columns;
			}

			inline bool isPenDown() const
			{
				if (m_a0 < 0 || m_a0 >= m_columns)
					return false;
				return m_data[m_a0 / 8] & (0x80 >> (m_a0 % 8));
			}

			inline void setA0(int a0)
			{
				m_a0 = a0;
			}
		
		private:
			int m_columns;
			int m_a0;
			std::vector<unsigned char> m_data;
		};

		bool m_encodedByteAlign;
		size_t m_columns;
		size_t m_rows;
		bool m_endOfBlock;
		bool m_blackIs1;
		DecodingLine m_referenceLine;
		DecodingLine m_codingLine;
	};
}

#endif
