#include <algorithm>
#include "exceptions.h"
#include "deflatestreambitreader.h"

LemonSqueezy::DeflateStreamBitReader::DeflateStreamBitReader(std::istream &in)
  : StreamBitReader(in)
{
}

bool LemonSqueezy::DeflateStreamBitReader::appendNextBit(BitSequence &seq)
{
	if (m_bitPosition == 8 && !readNextByte())
		throw IOException("Unexpected end of stream encountered while reading Huffman code");
	
	seq.code <<= 1;
	seq.code |= (m_currentByte & (1U << m_bitPosition)) ? 1 : 0;
	seq.length++;
	m_bitPosition++;
	return true;
}

bool LemonSqueezy::DeflateStreamBitReader::readBitsIntoBitSequence(BitSequence::size_type count, BitSequence &seq)
{
	if (count == 0)
		return true;

	if (seq.length + count > sizeof(seq.code) * CHAR_BIT)
		throw std::out_of_range("bit sequence has reached maximum length");

	if (seq.length)
		seq.code <<= count;
	
	size_t bitsLeft = count;
	while (bitsLeft > 0)
	{
		if (m_bitPosition == CHAR_BIT && !readNextByte())
			throw IOException("Unexpected end of stream encountered while reading bit sequence");

		size_t bitsFromThisByte = std::min(CHAR_BIT - m_bitPosition, bitsLeft);
		size_t mask = (1U << bitsFromThisByte) - 1;
		seq.code |= ((m_currentByte >> m_bitPosition) & mask) << (count - bitsLeft);
		
		bitsLeft -= bitsFromThisByte;
		m_bitPosition += bitsFromThisByte;
	}

	seq.length += count;
	return true;
}
