#include <algorithm>
#include "deflatestreambitwriter.h"
#include "exceptions.h"

LemonSqueezy::DeflateStreamBitWriter::DeflateStreamBitWriter(std::ostream &out)
  : StreamBitWriter(out)
{
}

void LemonSqueezy::DeflateStreamBitWriter::writeBits(const BitSequence &bs)
{
	size_t bitsLeft = bs.length;
	while (bitsLeft > 0)
	{
		size_t bitsIntoThisByte = std::min(8 - m_bitPosition, bitsLeft);
		size_t mask = (1U << bitsIntoThisByte) - 1;

		m_currentByte |= ((bs.code >> (bs.length - bitsLeft) & mask) << m_bitPosition);

		m_bitPosition += bitsIntoThisByte;
		bitsLeft -= bitsIntoThisByte;

		if (m_bitPosition == 8)
			ejectByte();
	}
}

void LemonSqueezy::DeflateStreamBitWriter::writeBitsBackwards(const BitSequence &bs)
{
	size_t bitsLeft = bs.length;

	while (bitsLeft > 0)
	{
		size_t bitsIntoThisByte = 8 - m_bitPosition;
		if (bitsIntoThisByte > bitsLeft)
			bitsIntoThisByte = bitsLeft;
		
		for (size_t i = 0; i < bitsIntoThisByte; i++)
			m_currentByte |= ((bs.code >> (bitsLeft-- - 1) & 0x1) << (m_bitPosition++));

		if (m_bitPosition == 8)
			ejectByte();
	}
}
