#include "rollingoutputbuffer.h"

LemonSqueezy::RollingOutputBuffer::RollingOutputBuffer(size_t bufferSize, std::ostream &out)
  : m_bufferSize(bufferSize), m_position(0), m_output(out)
{
	m_buffer.reserve(m_bufferSize);
}

LemonSqueezy::RollingOutputBuffer::~RollingOutputBuffer()
{
	flush();
}

void LemonSqueezy::RollingOutputBuffer::flush()
{
	m_output.write(reinterpret_cast<const char *>(&m_buffer[m_position]), m_buffer.size() - m_position);
	m_output.write(reinterpret_cast<const char *>(&m_buffer[0]), m_position);
}

void LemonSqueezy::RollingOutputBuffer::put(unsigned char byte)
{
	if (m_buffer.size() < m_bufferSize)
	{
		m_buffer.push_back(byte);
	}
	else
	{
		m_output.put(m_buffer[m_position]);
		m_buffer[m_position] = byte;
	}

	m_position = (m_position + 1) % m_bufferSize;
}

void LemonSqueezy::RollingOutputBuffer::repeatBytes(size_t backwardsDistance, size_t numberOfBytes)
{
	if (backwardsDistance > m_buffer.size())
		throw RollingOutputBufferException("Backwards distance exceeds available bytes");
	
	// have to be careful about how we calculate the first byte
	// since just doing m_position - backwardsDistance could result in
	// underflow
	size_t byteOffset = (m_position + m_bufferSize - backwardsDistance) % m_bufferSize;

	if (byteOffset >= m_buffer.size())
		throw RollingOutputBufferException("Internal inconsistency: first byte offset exceeds existing data count");
	
	while (numberOfBytes--)
	{
		unsigned char byte = m_buffer[byteOffset];
		put(byte);
		byteOffset = (byteOffset + 1) % m_buffer.size();
	}
}
