#include <algorithm>
#include "streambitwriter.h"

const size_t LemonSqueezy::StreamBitWriter::g_bufferSize = 8192;

LemonSqueezy::StreamBitWriter::StreamBitWriter(std::ostream &out)
  : m_bitPosition(0), m_currentByte(0), m_out(out), m_buffer(new unsigned char[g_bufferSize]), m_bufferPosition(0)
{
}

LemonSqueezy::StreamBitWriter::~StreamBitWriter()
{
	// force the byte out if we are not at a byte boundary
	alignToByteBoundary();
	flush();
	delete[] m_buffer;
}

void LemonSqueezy::StreamBitWriter::alignToByteBoundary()
{
	if (m_bitPosition != 0)
		ejectByte();
}

void LemonSqueezy::StreamBitWriter::ejectByte()
{
	m_buffer[m_bufferPosition++] = m_currentByte;
	if (m_bufferPosition == g_bufferSize)
		flush();
	m_currentByte = 0;
	m_bitPosition = 0;
}

void LemonSqueezy::StreamBitWriter::flush()
{
	if (m_bufferPosition > 0)
	{
		m_out.rdbuf()->sputn(reinterpret_cast<const char *>(m_buffer), m_bufferPosition);
		m_bufferPosition = 0;
	}
}

size_t &LemonSqueezy::StreamBitWriter::getCurrentBitPosition()
{
	return m_bitPosition;
}

unsigned char &LemonSqueezy::StreamBitWriter::getCurrentByte()
{
	return m_currentByte;
}

void LemonSqueezy::StreamBitWriter::writeBits(const BitSequence &bits)
{
	size_t bitsLeft = bits.length;

	while (bitsLeft > 0)
	{
		size_t bitsIntoThisByte = std::min(8 - m_bitPosition, bitsLeft);
		size_t mask = (1 << bitsIntoThisByte) - 1;
		m_currentByte |= ((bits.code >> (bitsLeft - bitsIntoThisByte)) & mask) << ((8 - m_bitPosition) - bitsIntoThisByte);

		bitsLeft -= bitsIntoThisByte;
		m_bitPosition += bitsIntoThisByte;
		if (m_bitPosition == 8)
			ejectByte();
	}
}
