#ifndef STRIPEDHORSE_XPSDOCUMENT_H
#define STRIPEDHORSE_XPSDOCUMENT_H

#include <memory>
#include <string>
#include <lemonsqueezy/zip.h>
#include "exceptions.h"
#include "page.h"
#include "pageelementcontext.h"

namespace LemonXPS
{
	class Document;

	using DocumentPtr = std::shared_ptr<Document>;
	using DocumentCPtr = std::shared_ptr<const Document>;
	using DocumentWeakPtr = std::weak_ptr<Document>;

	class Document
	{
	public:
		static DocumentPtr Load(const LemonSqueezy::ZipArchive &zip, const std::string &source);
		~Document();

		const LemonSqueezy::ZipArchive &getArchive() const;
		const std::vector<PagePtr> &getPages() const;
		const std::string &getSource() const;
		void render(IPageElementContext &) const;

	private:
		Document(const LemonSqueezy::ZipArchive &archive, const std::string &source);
		LemonSqueezy::ZipArchive m_archive;
		std::string m_source;
		std::vector<PagePtr> m_pages;
	};

	class XPSDocumentException : public XPSException
	{
	public:
		XPSDocumentException(const std::string &msg) : XPSException(msg) {}
	};
}

#endif