#include "libxmlhelper.h"

namespace
{
	class LibXml2Helper
	{
	public:
		LibXml2Helper()
		{
			LIBXML_TEST_VERSION;
		}

		~LibXml2Helper()
		{
			xmlCleanupParser();
		}
	};

	// automatically calls xmlCleanupParser() on exit
	LibXml2Helper g_helper;
}

xmlDocPtr LemonXPS::XML::CreateXMLDocFromZipArchiveEntry(const LemonSqueezy::ZipArchiveEntry &zae)
{
	const std::vector<unsigned char> &data = zae.decompress();
	xmlDocPtr doc = xmlReadMemory(reinterpret_cast<const char *>(&data.at(0)), data.size(), zae.getFilename().c_str(), nullptr, 0);
	return doc;
}

void LemonXPS::XML::EnumerateProperties(xmlNodePtr node, std::function<void(const std::string &, const std::string &)> callback)
{
	for (xmlAttrPtr p = node->properties; p != nullptr; p = p->next)
	{
		std::string name = GetName(p);

		xmlChar *attrValue = xmlNodeListGetString(node->doc, p->children, 1);
		std::string value { reinterpret_cast<const char *>(attrValue) };
		xmlFree(attrValue);

		callback(name, value);
	}
}

xmlNodePtr LemonXPS::XML::GetFirstElementChild(xmlNodePtr node)
{
	for (xmlNodePtr child = node->children; child != nullptr; child = child->next)
		if (child->type == XML_ELEMENT_NODE)
			return child;
	return nullptr;
}

std::string LemonXPS::XML::GetName(xmlAttrPtr attr)
{
	return std::string { reinterpret_cast<const char *>(attr->name) };
}

std::string LemonXPS::XML::GetName(xmlNodePtr node)
{
	return std::string { reinterpret_cast<const char *>(node->name) };
}

std::string LemonXPS::XML::GetNamespace(xmlNodePtr node)
{
	if (node->ns != nullptr)
		return std::string { reinterpret_cast<const char *>(node->ns->href) };
	else
		return std::string();
}

xmlNodePtr LemonXPS::XML::GetNextElementSibling(xmlNodePtr node)
{
	for (xmlNodePtr sibling = node->next; sibling != nullptr; sibling = sibling->next)
		if (sibling->type == XML_ELEMENT_NODE)
			return sibling;
	return nullptr;
}

std::string LemonXPS::XML::GetProperty(xmlNodePtr node, const std::string &propertyName)
{
	std::string value;
	(void) TryGetProperty(node, propertyName, value);
	return value;
}

bool LemonXPS::XML::HasProperty(xmlNodePtr node, const std::string &propertyName)
{
	return xmlHasProp(node, BAD_CAST propertyName.c_str());
}

bool LemonXPS::XML::TryGetProperty(xmlNodePtr node, const std::string &name, std::string &value)
{
	xmlChar *propertyValue = xmlGetProp(node, BAD_CAST name.c_str());
	if (propertyValue == nullptr)
	{
		value = "";
		return false;
	}

	value = reinterpret_cast<const char *>(propertyValue);
	xmlFree(propertyValue);
	return true;
}