#ifndef LEMONXPS_XPSPAGE_H
#define LEMONXPS_XPSPAGE_H

#include <memory>
#include <string>
#include <libcore/types.h>
#include "exceptions.h"
#include "pageelement.h"
#include "pageelementcontext.h"

namespace LemonXPS
{
	class Document;

	class Page;
	using PagePtr = std::shared_ptr<Page>;
	using PageCPtr = std::shared_ptr<const Page>;

	class Page : public std::enable_shared_from_this<Page>
	{
	public:
		static PagePtr Load(std::shared_ptr<Document> document, const std::string &root, const std::string &source);
		~Page();

		std::weak_ptr<Document> getDocument() const;
		double getHeight() const;
		PageElementPtr getRoot();
		double getWidth() const;
		void render(IPageElementContext &ctx) const;

	private:
		Page(std::shared_ptr<Document> document, const std::string &root, const std::string &source);
		std::weak_ptr<Document> m_document;
		std::string m_root;
		std::string m_source;
		PageElementPtr m_rootElement;
	};

	class XPSPageException : public XPSException
	{
	public:
		XPSPageException(const std::string &msg) : XPSException(msg) {}
	};
}

#endif
