#include "page.h"
#include "pageelement.h"

LemonXPS::PageElement::PageElement(PagePtr page, const std::string &type, const std::string &ns, const Attributes &attr)
  : m_type(type), m_namespace(ns), m_attributes(attr), m_page(page)
{
}

LemonXPS::PageElement::~PageElement()
{
}

void LemonXPS::PageElement::addChild(PageElementPtr p)
{
	m_children.emplace_back(p);
}

std::string LemonXPS::PageElement::getAttribute(const std::string &name) const
{
	std::string result;
	tryGetAttribute(name, result);
	return result;
}

const LemonXPS::Attributes &LemonXPS::PageElement::getAttributes() const
{
	return m_attributes;
}


const std::vector<LemonXPS::PageElementPtr> &LemonXPS::PageElement::getChildren() const
{
	return m_children;
}

const std::string &LemonXPS::PageElement::getNamespace() const
{
	return m_namespace;
}

std::weak_ptr<LemonXPS::Page> LemonXPS::PageElement::getPage() const
{
	return m_page;
}

const std::string &LemonXPS::PageElement::getType() const
{
	return m_type;
}

bool LemonXPS::PageElement::hasAttribute(const std::string &name) const
{
	return m_attributes.find(name) != m_attributes.end();
}

bool LemonXPS::PageElement::tryGetAttribute(const std::string &name, std::string &value) const
{
	Attributes::const_iterator i = m_attributes.find(name);
	if (i == m_attributes.end())
	{
		value = "";
		return false;
	}
	
	value = i->second;
	return true;
}
