#ifndef LEMONXPS_PAGEELEMENT_H
#define LEMONXPS_PAGEELEMENT_H

#include <memory>
#include <string>
#include <unordered_map>
#include <vector>

namespace LemonXPS
{
	class Page;

	class PageElement;
	using PageElementPtr = std::shared_ptr<PageElement>;
	using PageElementCPtr = std::shared_ptr<const PageElement>;

	typedef std::unordered_map<std::string, std::string> Attributes;
	
	class PageElement
	{
	public:
		PageElement(std::shared_ptr<Page> page, const std::string &type, const std::string &ns, const Attributes &attr);
		virtual ~PageElement();

		void addChild(PageElementPtr p);
		std::string getAttribute(const std::string &name) const;
		const Attributes &getAttributes() const;
		const std::vector<PageElementPtr> &getChildren() const;
		const std::string &getNamespace() const;
		std::weak_ptr<Page> getPage() const;
		const std::string &getType() const;
		bool hasAttribute(const std::string &name) const;
		bool hasAttributes(const std::initializer_list<std::string> &attr) const;
		bool tryGetAttribute(const std::string &name, std::string &value) const;

	private:
		std::string m_type;
		std::string m_namespace;
		Attributes m_attributes;
		std::weak_ptr<Page> m_page;
		std::vector<PageElementPtr> m_children;
	};
}

#endif
