#include <cctype>
#include <tuple>
#include "pathparser.h"

namespace
{
	bool getNextOperator(std::istream &in, char &op)
	{
		std::istream::int_type ch;
		while ((ch = in.get()) != std::istream::traits_type::eof())
		{
			op = static_cast<char>(ch);
			
			if (::isspace(op))
				continue;
			
			if (::isalpha(op))
				return true;
			
			throw std::invalid_argument("not a valid abbreviated path");
		}

		return false;
	}

	std::tuple<double, double> parseCoordinates(std::istream &in)
	{
		double c1, c2;
		char delim;

		if ((in >> c1 >> delim >> c2) && delim == ',')
			return std::make_tuple(c1, c2);
		
		throw std::invalid_argument("unable to parse coordinates");
	}
}

LemonXPS::PathParser::PathParser(std::istream &in)
  : m_input(in)
{
}

void LemonXPS::PathParser::render(IPathParserContext &context)
{
	char op;
	while (getNextOperator(m_input, op))
	{
		switch (op)
		{
			case 'C':
			{
				double cx1, cy1, cx2, cy2, x3, y3;
				std::tie(cx1, cy1) = parseCoordinates(m_input);
				std::tie(cx2, cy2) = parseCoordinates(m_input);
				std::tie(x3, y3) = parseCoordinates(m_input);
				context.curveTo(cx1, cy1, cx2, cy2, x3, y3);
				break;
			}

			case 'L':
			{
				double x, y;
				std::tie(x, y) = parseCoordinates(m_input);
				context.lineTo(x, y);
				break;
			}

			case 'M':
			{
				double x, y;
				std::tie(x, y) = parseCoordinates(m_input);
				context.moveTo(x, y);
				break;
			}

			case 'Z':
			{
				context.closePath();
				break;
			}

			default:
				throw std::invalid_argument("Unknown path operator");
		}
	}
}
