#include <array>
#include <fstream>
#include "logging.h"

namespace
{
	std::string g_debug = "Debug";
	std::string g_info = "Info";
	std::string g_warning = "Warning";
	std::string g_error = "Error";
	std::string g_critical = "Critical";

	std::ofstream g_nullStream; // if it is never opened, all write operations will simply fail
	std::ostream *g_output = &std::cerr;

	std::ostream &prepareLog(const std::string &prefix)
	{
		if (prefix.size() > 0)
			(*g_output) << prefix << ": ";
		return *g_output;
	}
}

void StripedHorse::Logging::SetOutputStream(std::ostream *out)
{
	if (out)
		g_output = out;
	else
		g_output = &g_nullStream;
}

void StripedHorse::Logging::SetMessagePrefixes(const std::string &debug, const std::string &info, const std::string &warning, const std::string &error, const std::string &critical)
{
	g_debug = debug;
	g_info = info;
	g_warning = warning;
	g_error = error;
	g_critical = critical;
}

std::ostream &StripedHorse::Logging::Critical()
{
	return prepareLog(g_critical);
}

std::ostream &StripedHorse::Logging::Debug()
{
	return prepareLog(g_debug);
}

std::ostream &StripedHorse::Logging::Error()
{
	return prepareLog(g_error);
}

std::ostream &StripedHorse::Logging::Info()
{
	return prepareLog(g_info);
}

std::ostream &StripedHorse::Logging::Warning()
{
	return prepareLog(g_warning);
}
