#ifndef STRIPEDHORSE_STREAMS_H
#define STRIPEDHORSE_STREAMS_H

#include <iostream>
#include <vector>

namespace StripedHorse
{
	template<typename CharT = char, typename VectorElemT = unsigned char>
	class BasicInputVectorStreamBuffer : public std::streambuf
	{
	public:
		using char_type = CharT;

		// clumsy casts required due to C++ iostreams interfaces
		BasicInputVectorStreamBuffer(const std::vector<VectorElemT> &vector)
		  : m_start(reinterpret_cast<char_type *>(const_cast<VectorElemT *>(vector.data()))), m_end(m_start + vector.size())
		{
			setg(m_start, m_start, m_end);
		}

	protected:
		pos_type seekoff(off_type off, std::ios_base::seekdir dir, std::ios_base::openmode which = std::ios_base::in) override
		{
			if (which == std::ios_base::in)
			{
				if (dir == std::ios_base::cur)
					gbump(off);
				else if (dir == std::ios_base::end)
					setg(eback(), egptr() + off, egptr());
				else if (dir == std::ios_base::beg)
					setg(eback(), eback() + off, egptr());
				return gptr() - eback();
			}
			else
			{
				return pptr() - pbase();
			}
		}

		pos_type seekpos(pos_type newpos, std::ios_base::openmode which) override
		{
			return seekoff(newpos, std::ios_base::beg, which);
		}
	
	private:
		char_type *m_start;
		char_type *m_end;
	};

	template<typename CharT = char, typename VectorElemT = unsigned char>
	class BasicOutputVectorStreamBuffer : public std::basic_streambuf<CharT>
	{
	public:
		using char_type = CharT;
		using vector_elem_type = VectorElemT;
		using vector_type = std::vector<vector_elem_type>;
		using int_type = typename std::basic_streambuf<CharT>::int_type;
		using traits_type = typename std::basic_streambuf<CharT>::traits_type;

		BasicOutputVectorStreamBuffer(vector_type &vector)
		  : m_vector(vector)
		{
		}

	protected:
		// allow bulk writes to the stream (otherwise overflow() is called for
		// each character in the stream).
		std::streamsize xsputn(const CharT *ch, std::streamsize n) override
		{
			m_vector.insert(m_vector.end(), ch, ch + n);
			return n;
		}

		int_type overflow(int_type ch) override
		{
			m_vector.push_back(static_cast<VectorElemT>(traits_type::to_char_type(ch)));
			return ch;
		}
	
	private:
		vector_type &m_vector;
	};
}

#endif
