#include <algorithm>
#include <cstdlib>
#include <iostream>
#include <sstream>
#include "uuid.h"

StripedHorse::UUID::UUID()
  : m_bytes { }
{
}

StripedHorse::UUID::UUID(const std::array<uint8_t, 16> &data)
  : m_bytes { data }
{
}

StripedHorse::UUID::UUID(const UUID &other)
  : m_bytes { other.m_bytes }
{
}

StripedHorse::UUID::UUID(UUID &&other)
  : m_bytes { other.m_bytes }
{
}

StripedHorse::UUID StripedHorse::UUID::Derive(const std::vector<unsigned char> &data)
{
	UUID result;

	if (data.size() == 0)
		return result;

	const size_t uuidsize = result.m_bytes.size();
	
	if (data.size() < uuidsize)
	{
		for (size_t i = 0; i < uuidsize; i++)
			result.m_bytes.at(i) = ~data.at(i % data.size());
	}
	else
	{
		for (size_t i = 0; i < data.size(); i++)
			result.m_bytes.at(i % uuidsize) ^= data.at(i);
	}
	
	return result;
}

StripedHorse::UUID StripedHorse::UUID::GenerateRandom()
{
	UUID result;

	std::cerr << "Warning: generated UUID is not a valid UUID" << std::endl;
	
	for (size_t i = 0; i < result.m_bytes.size(); i++)
		result.m_bytes[i] = rand();
	
	return result;
}

const std::array<uint8_t, 16> &StripedHorse::UUID::getBytes() const
{
	return m_bytes;
}

StripedHorse::UUID::operator std::string() const
{
	std::ostringstream result;

	result.fill('0');
	result.setf(std::ios_base::hex, std::ios_base::basefield);
	
	size_t order[] = { 3, 2, 1, 0, 5, 4, 7, 6, 8, 9, 10, 11, 12, 13, 14, 15 };

	for (size_t i = 0; i < m_bytes.size(); i++)
	{
		result.width(2);
		result << static_cast<int>(m_bytes[order[i]]);
		if (i == 3 || i == 5 || i == 7 || i == 9)
			result << '-';
	}
		
	return result.str();
}
