#include <cstdlib>
#include <vector>
#include <libcore/exceptions.h>
#include "files.h"

#ifndef _WIN32
#include <unistd.h>

std::string StripedHorse::Files::GetTemporaryFilename(const std::string &prefix)
{
	std::string tempfilename;
	char *tmpDir = getenv("TMPDIR");
	if (tmpDir)
		tempfilename = tmpDir;
	else
		tempfilename = "/tmp";
	
	tempfilename += "/";
	tempfilename += prefix;
	tempfilename += "-XXXXXX";
	
	std::vector<char> filenameBuffer { tempfilename.begin(), tempfilename.end() };
	filenameBuffer.push_back('\0');

	int tempfd = mkstemp(&filenameBuffer[0]);
	if (tempfd == -1)
		throw StripedHorse::IOException("Unable to get temporary file: " + tempfilename);
	
	close(tempfd);
	return std::string { &filenameBuffer[0] };
}

bool StripedHorse::Files::SetStandardInputToBinaryMode()
{
	// no need to do this on POSIX systems
	return true;
}

bool StripedHorse::Files::SetStandardOutputToBinaryMode()
{
	// no need to do this on POSIX systems
	return true;
}

#else
#include <fcntl.h>
#include <io.h>
#include <windows.h>
#define unlink _unlink

std::string StripedHorse::Files::GetTemporaryFilename(const std::string &prefix)
{
	// GetTempPath can apparently write MAX_PATH + 1 characters
	std::vector<char> tempPathBuffer(MAX_PATH + 2);
	DWORD tempPathLength = GetTempPathA(tempPathBuffer.size(), tempPathBuffer.data());

	if (tempPathLength == 0)
		throw StripedHorse::IOException("Unable to determine temporary directory");

	// GetTempFileName writes only MAX_PATH (including null terminator)
	std::vector<char> tempFileNameBuffer(MAX_PATH);
	UINT result = GetTempFileNameA(tempPathBuffer.data(), prefix.c_str(), 0, tempFileNameBuffer.data());

	if (result == 0)
		throw StripedHorse::IOException("Unable to generate temporary filename");
	
	return std::string { tempFileNameBuffer.data() };
}

bool StripedHorse::Files::SetStandardInputToBinaryMode()
{
	return _setmode(_fileno(stdin), _O_BINARY) != -1;
}

bool StripedHorse::Files::SetStandardOutputToBinaryMode()
{
	return _setmode(_fileno(stdout), _O_BINARY) != -1;
}

#endif
