#ifdef _WIN32
#include <windows.h>
#define unlink _unlink
#else
#include <unistd.h>
#endif

#include <iostream>
#include <fstream>
#include <libcore/logging.h>
#include "files.h"
#include "managedfile.h"

void StripedHorse::ManagedFile::SetManagedFileToStandardInput(ManagedFile &managedFile, const std::string &tempPrefix)
{
	managedFile.m_filename = StripedHorse::Files::GetTemporaryFilename(tempPrefix);
	managedFile.m_deleteWhenDone = true;

	if (!Files::SetStandardInputToBinaryMode())
		StripedHorse::Logging::Warning() << "Unable to set standard input to binary mode" << std::endl;

	std::ofstream output(managedFile.m_filename, std::ios_base::out | std::ios_base::binary);
	if (!output.is_open())
	{
		StripedHorse::Logging::Critical() << "Unable to open temporary file" << std::endl;
		::exit(EXIT_FAILURE);
	}
	output << std::cin.rdbuf();
}

StripedHorse::ManagedFile::ManagedFile()
  : m_deleteWhenDone(false)
{
}

StripedHorse::ManagedFile::ManagedFile(const std::string &filename, bool deleteWhenDone)
  : m_filename(filename), m_deleteWhenDone(deleteWhenDone)
{
}

StripedHorse::ManagedFile::ManagedFile(ManagedFile &&other)
  : m_filename(std::move(other.m_filename)), m_deleteWhenDone(other.m_deleteWhenDone)
{
	other.m_deleteWhenDone = false;
}

StripedHorse::ManagedFile::~ManagedFile()
{
	if (m_deleteWhenDone && !m_filename.empty())
		unlink(m_filename.c_str());
}

const std::string &StripedHorse::ManagedFile::getFilename() const
{
	return m_filename;
}

void StripedHorse::ManagedFile::setDeleteWhenDone(bool b)
{
	m_deleteWhenDone = b;
}

void StripedHorse::ManagedFile::setFilename(const std::string &filename)
{
	m_filename = filename;
}
