#ifndef STRIPEDHORSE_PPD_H
#define STRIPEDHORSE_PPD_H

#include <iostream>
#include <string>
#include <unordered_map>
#include <vector>

#include <libcore/exceptions.h>

#include "ppdvalue.h"
#include "ppdstatement.h"

namespace StripedHorse
{
	class PPDFile
	{
	public:
		typedef std::unordered_multimap<std::string, PPDStatement> PPDStatementMap;

		PPDFile();
		PPDFile(std::istream &);
		PPDFile(const std::string &);
		PPDFile(const PPDFile &);
		PPDFile(PPDFile &&);
		PPDFile &operator=(const PPDFile &);
		
		
		PPDStatementMap::const_iterator begin() const;
		size_t countStatementsWithMainKeyword(const std::string &) const;
		PPDStatementMap::const_iterator end() const;
		std::vector<PPDStatement> getAllStatementsWithMainKeyword(const std::string &) const;
		const PPDStatement &getAnyStatementWithMainKeyword(const std::string &) const;
		std::string getDefaultChoiceForOption(const std::string &optionName) const;
		bool hasStatementWithMainKeyword(const std::string &) const;
		bool hasOption(const std::string &optionName) const;
		bool hasOptionWithChoice(const std::string &optionName, const std::string &choiceName) const;
		void load(std::istream &);
		bool tryGetAnyStatementWithMainKeyword(const std::string &, PPDStatement &) const;
		
	private:
		PPDStatementMap m_statements;
	};
	
	class PPDException : public Exception
	{
	public:
		PPDException(const std::string &msg) : Exception(msg) {}
	};
	
	class PPDKeywordNotFoundException : public PPDException
	{
	public:
		PPDKeywordNotFoundException(const std::string &msg) : PPDException(msg) {}
	};
	
	class PPDSyntaxException : public PPDException
	{
	public:
		PPDSyntaxException(size_t lineNumber, const std::string &msg) : PPDException(msg), m_lineNumber(lineNumber) {}
		size_t getLineNumber() const { return m_lineNumber; }
	
	private:
		size_t m_lineNumber;
	};
}

#endif
