#include "ppdstatement.h"

using namespace StripedHorse;

PPDStatement::PPDStatement()
  : m_lineNumber(0)
{
}

PPDStatement::PPDStatement(size_t ln, const std::string &main)
  : m_lineNumber(ln), m_mainKeyword(main)
{
}

PPDStatement::PPDStatement(size_t ln, const std::string &main, const PPDValue &val)
  : m_lineNumber(ln), m_mainKeyword(main), m_value(val)
{
}

PPDStatement::PPDStatement(size_t ln, const std::string &main, const std::string &option, const PPDValue &val)
  : m_lineNumber(ln), m_mainKeyword(main), m_optionKeyword(option),
    m_value(val)
{
}

PPDStatement::PPDStatement(size_t ln, const std::string &main, const std::string &option, const std::string &translation, const PPDValue &val)
  : m_lineNumber(ln), m_mainKeyword(main), m_optionKeyword(option),
    m_translation(translation), m_value(val)
{
}

PPDStatement::PPDStatement(const PPDStatement &other)
{
	this->operator=(other);
}

PPDStatement::PPDStatement(PPDStatement &&other)
  : m_lineNumber(other.m_lineNumber),
    m_mainKeyword(std::move(other.m_mainKeyword)),
    m_optionKeyword(std::move(other.m_optionKeyword)),
    m_translation(std::move(other.m_translation)),
    m_value(std::move(other.m_value))
{
}

PPDStatement &PPDStatement::operator=(const PPDStatement &other)
{
	m_lineNumber = other.m_lineNumber;
	m_mainKeyword = other.m_mainKeyword;
	m_optionKeyword = other.m_optionKeyword;
	m_translation = other.m_translation;
	m_value = other.m_value;
	return *this;
}

size_t PPDStatement::getLineNumber() const
{
	return m_lineNumber;
}

const std::string &PPDStatement::getMainKeyword() const
{
	return m_mainKeyword;
}

const std::string &PPDStatement::getOptionKeyword() const
{
	return m_optionKeyword;
}

const std::string &PPDStatement::getTranslation() const
{
	return m_translation;
}

const PPDValue &PPDStatement::getValue() const
{
	return m_value;
}

bool PPDStatement::hasOptionKeyword() const
{
	return m_optionKeyword.length() > 0;
}

bool PPDStatement::hasTranslation() const
{
	return m_translation.length() > 0;
}

bool PPDStatement::hasValue() const
{
	return m_value.getType() != PPDValue::ValueType::kNoValue;
}
