#include <cctype>
#include <fstream>
#include <regex>
#include <unordered_map>

#include "string-escape.h"

namespace
{
	int unhex(char c)
	{
		c = ::tolower(c);
		if (c >= '0' && c <= '9')
			return c - '0';
		else if (c >= 'a'  && c <= 'f')
			return c - 'a' + 10;
		else
			return 0;
	}
	
	char tohex(int i)
	{
		if (i < 10)
			return '0' + i;
		else if (i >= 10 && i < 16)
			return i - 10 + 'a';
		else
			return 'x';
	}
}

std::string StripedHorse::UnescapeString(const std::string &input)
{
	static const std::unordered_map<std::string, std::string> unescapes
	{
		{ "\\0", std::string(1, '\0') }, // needs special treatment
		{ "\\\\", "\\" },
		{ "\\\"", "\"" },
		{ "\\a", "\a" },
		{ "\\b", "\b" },
		{ "\\e", "\x1B" },
		{ "\\f", "\f" },
		{ "\\n", "\n" },
		{ "\\r", "\r" },
		{ "\\t", "\t" },
		{ "\\v", "\v" }
	};
	
	static const std::regex escapeRegex("\\\\(x[0-9A-Fa-f]{2}|[^x])");
	
	std::string output;
	std::sregex_iterator stringBegin(input.begin(), input.end(), escapeRegex);
	std::sregex_iterator stringEnd;
	
	std::string::size_type lastEscape = 0;
	
	for (std::sregex_iterator iter = stringBegin; iter != stringEnd; iter++)
	{
		const std::string &escape = iter->str();
		std::string escaped;
		
		if (escape[1] == 'x')
		{
			char unhexed = unhex(escape[2]) << 4 | unhex(escape[3]);
			escaped.push_back(unhexed);
		}
		else
		{
			auto replacement = unescapes.find(escape);
			if (replacement == unescapes.end())
				throw UnknownEscapeException(escape);
			escaped = replacement->second;
		}
		
		output.append(input.substr(lastEscape, iter->position() - lastEscape));
		output.append(escaped);
		
		lastEscape = iter->position() + iter->length();
	}
	
	output.append(input.substr(lastEscape));
	
	return output;
}

std::string StripedHorse::EscapeString(const std::string &input)
{
	static const std::unordered_map<char, std::string> g_escapes
	{
		{ '\0', "\\0" },
		{ '\\', "\\\\" },
		{ '\"', "\\\"" },
		{ '\a', "\\a" },
		{ '\b', "\\b" },
		{ '\x1B', "\\e" },
		{ '\f', "\\f" },
		{ '\n', "\\n" },
		{ '\r', "\\r" },
		{ '\t', "\\t" },
		{ '\v', "\\v" }
	};
	
	std::string result;
	result.reserve(input.size());
	
	for (auto ch : input)
	{
		unsigned char uch = ch;
		auto knownEscape = g_escapes.find(ch);
		if (knownEscape != g_escapes.end())
		{
			result.append(knownEscape->second);
		}
		else if (uch > 127)
		{
			result.append("\\x");
			result.push_back(tohex((uch >> 4) & 0xF));
			result.push_back(tohex(uch & 0xF));
		}
		else
		{
			result.push_back(ch);
		}
	}
	
	return result;
}
