#ifndef STRIPEDHORSE_STRING_ESCAPE_H

#include <string>
#include <libcore/exceptions.h>

namespace StripedHorse
{
	/**
	 This function takes an input string and escapes certain characters by
	 preceding them with a backslash or by replacing them with an escape
	 sequence. The result is similar to how the string would appear in C
	 source code.
	 @param[in] unescaped the string to perform the escaping on
	 @result if the string contains characters requiring escape sequences (such
	         as newline characters, tabs, the null character etc) then the
	         result is the input string with all the appropriate characters
	         either escaped with a backslash or replaced with an escape
	         sequence (depending on the character). If no characters required
	         escaping, the result is a copy of the input.
	 */
	std::string EscapeString(const std::string &unescaped);
	
	/**
	 This function interprets escape sequences and escaped characters in a
	 string and returns the string with all escape sequences and escaped
	 characters replaced with their actual values.
	 @param[in] escaped the string that contains escape sequences
	 @result if the string contains escape sequences or escaped characters, the
	         result is the input string with all escape sequences or escaped
	         characters replaced with their actual values. For example, the
	         two character escape sequence "\n" is replaced with a newline
	         character.
	 */
	std::string UnescapeString(const std::string &escaped);
	
	class UnknownEscapeException : public Exception
	{
	public:
		UnknownEscapeException(const std::string &msg) : Exception(msg) {}
	};
}

#endif
