#ifndef STRIPEDHORSE_SHAPEDETECTION_H
#define STRIPEDHORSE_SHAPEDETECTION_H

#include <memory>
#include <lemonpdf/gfxpath.h>
#include <lemonpdf/gfxcolor.h>

namespace StripedHorse
{
	class DetectedShape;
	using DetectedShapePtr = std::shared_ptr<DetectedShape>;

	enum class FixedColor
	{
		kRed,
		kYellow,
		kGreen,
		kCyan,
		kBlue,
		kMagenta,
		kOther
	};

	class DetectedShape
	{
	public:
		static DetectedShapePtr DetectFromPath(const LemonPDF::Graphics::Path &p, LemonPDF::Graphics::ColorPtr color, double hueThresholdDegrees = 5.0, double saturationThreshold = 0.1, double valueThreshold = 0.1);

		DetectedShape(enum FixedColor c);
		virtual ~DetectedShape() {}

		virtual bool appendData(const LemonPDF::Point &p, const std::string &data);
		virtual double getAngle() const;
		enum FixedColor getColor() const;
		std::string getColorName() const;
		virtual const std::string &getData() const;
		virtual std::string getName() const;
		virtual LemonPDF::Rectangle getRegion() const;
		virtual bool pointIsInShape(const LemonPDF::Point &p) const;
		virtual void setAngle(double angle);
		
	private:
		enum FixedColor m_color;
		double m_angle;
	};
}

#endif
