-include ../tools/vars.mk

VERSION:=$(shell ../tools/getReleaseInfo.py ../tools/release.xml version)
OSXARCH=x86-64
DARWIN_PKG=build/toshiba-tec-barcode-printer-drivers-$(VERSION)-$(OSXARCH).pkg
BUILDTOKEN=build/.buildtoken.darwin

.PHONY: all
all: $(DARWIN_PKG)

.darwin.$(ARCH).deps: ../drv/tpcl_desktop.drv ../drv/tpcl_industrial.drv ../drv/tpcl_mobile.drv ../drv/db-ea4d.drv
	@./mkppddep.sh darwin /Contents/Library/Printers/PPDs/Contents/Resources "$@"

-include .darwin.$(ARCH).deps

.PHONY: security-unlock
security-unlock:
	@if [ -n "$${SSH_CONNECTION}" ] && [ "$$(hostname)" = "dominica" ] && [ "$${USER}" = "macdev" ]; then \
		echo "This appears to be an integration build, attempting to unlock the keychain for signing..."; \
		security unlock -p $$(< ~macdev/.macdev_credentials ) ~/Library/Keychains/tap.keychain-db; \
	fi

##########
# DARWIN #
##########

DARWIN_SUBPKG=darwin.$(ARCH)/toshiba-tec-barcode-printer-drivers.pkg
DARWIN_CUPSDIR=darwin.$(ARCH)/Contents/etc/cups
DARWIN_FILTERDIR=darwin.$(ARCH)/Contents/usr/libexec/cups/filter
DARWIN_BINDIR=darwin.$(ARCH)/Contents/usr/local/bin
DARWIN_PPDDIR=darwin.$(ARCH)/Contents/Library/Printers/PPDs/Contents/Resources

DARWIN_FILTER_TARGETS=$(DARWIN_FILTERDIR)/tecfilter
ifeq ($(DRIVERTYPE),full)
	DARWIN_FILTER_TARGETS += $(DARWIN_FILTERDIR)/pdftoxps \
	                         $(DARWIN_FILTERDIR)/xpstotpcl \
	                         $(DARWIN_CUPSDIR)/toshiba-tec.convs \
	                         $(DARWIN_CUPSDIR)/toshiba-tec.types
endif

$(DARWIN_PKG): security-unlock $(DARWIN_FILTERDIR) $(DARWIN_BINDIR) $(DARWIN_CUPSDIR) $(DARWIN_PPDDIR) $(BUILDTOKEN) $(DARWIN_FILTER_TARGETS) \
    $(DARWIN_BINDIR)/tectool \
    $(DARWIN_FILTERDIR)/tecfilter \
    darwin.$(ARCH)/striped-horse.dist
	pkgbuild \
		--root darwin.$(ARCH)/Contents \
		--version $(VERSION) \
		--identifier com.toshiba.barcode-printing.drivers \
		--install-location / \
		--ownership recommended \
		--sign "Developer ID Installer: Toshiba (Australia) Pty Limited" \
		"$(DARWIN_SUBPKG)"
	productbuild \
		--distribution darwin.$(ARCH)/striped-horse.dist \
		--package-path darwin.$(ARCH) \
		--version $(VERSION) \
		--sign "Developer ID Installer: Toshiba (Australia) Pty Limited" \
		"$@"

darwin.$(ARCH)/striped-horse.dist: support/macos/striped-horse.dist.in
	sed -e 's/\$${VERSION}/$(VERSION)/' "$<" > "$@"

$(DARWIN_CUPSDIR) $(DARWIN_FILTERDIR) $(DARWIN_PPDDIR) $(DARWIN_BINDIR):
	mkdir -m 755 -p "$@"

$(DARWIN_CUPSDIR)/toshiba-tec.convs: ../drv/striped-horse.convs
$(DARWIN_CUPSDIR)/toshiba-tec.types: ../drv/striped-horse.types
$(DARWIN_FILTERDIR)/pdftoxps: ../pdftoxps/build.macos.$(ARCH)/pdftoxps-static
$(DARWIN_FILTERDIR)/tecfilter: ../tecfilter/build.macos.$(ARCH)/tecfilter-static
$(DARWIN_FILTERDIR)/xpstotpcl: ../xpstotpcl/build.macos.$(ARCH)/xpstotpcl-static
$(DARWIN_BINDIR)/tectool: ../tectool/build.macos.$(ARCH)/tectool-static

$(DARWIN_FILTER_TARGETS): security-unlock

$(DARWIN_PPDDIR)/%: ../drv/ppd/%
	@mkdir -m 755 -p $$(dirname "$@")
	grep -v "TECTarget" < "$<" > "$@"
	@chmod 644 "$@"

$(DARWIN_CUPSDIR)/toshiba-tec.convs $(DARWIN_CUPSDIR)/toshiba-tec.types:
	$(INSTALL) -m 644 "$^" "$@"

$(DARWIN_FILTERDIR)/tecfilter $(DARWIN_FILTERDIR)/pdftoxps $(DARWIN_FILTERDIR)/xpstotpcl $(DARWIN_BINDIR)/tectool:
	cp -p "$<" "$@"
	strip "$@"
	codesign -s "Developer ID Application: Toshiba (Australia) Pty Limited" -o library,kill,runtime "$@"

.PHONY: clean
clean:
	rm -rf darwin.$(ARCH)
	-rm -f .darwin.$(ARCH).deps
	rm -rf build

$(BUILDTOKEN):
	mkdir -p $(shell dirname "$(BUILDTOKEN)")
	date | cksum > "$@"
