-include ../tools/vars.mk

ifeq ($(ARCH), x86-64)
	DEBARCH=amd64
	RPMARCH=x86_64
else ifeq ($(ARCH), i686)
	DEBARCH=i386
	RPMARCH=$(ARCH)
else ifeq ($(ARCH), i386)
	DEBARCH=$(ARCH)
	RPMARCH=i686
else ifeq ($(ARCH), arm)
	DEBARCH=armhf
	RPMARCH=armv7l
endif

VERSION:=$(shell ../tools/getReleaseInfo.py ../tools/release.xml version)

INSTALL?=/usr/bin/install

DEBIAN_PKG=build/toshiba-tec-barcode-printer-drivers_$(VERSION)_debian_$(DEBARCH).deb
REDHAT_PKG=build/toshiba-tec-barcode-printer-drivers-$(VERSION)-1.$(RPMARCH).rpm

BUILDTOKEN=build/.buildtoken

.PHONY: all
all: all.$(OSNAME)

.PHONY: all.Linux
all.Linux: $(DEBIAN_PKG) $(REDHAT_PKG)

.debian.$(ARCH).deps: ../drv/tpcl_desktop.drv ../drv/tpcl_industrial.drv ../drv/tpcl_mobile.drv ../drv/db-ea4d.drv
	@./mkppddep.sh debian /usr/share/cups/model "$@"

.redhat.$(ARCH).deps: ../drv/tpcl_desktop.drv ../drv/tpcl_industrial.drv ../drv/tpcl_mobile.drv ../drv/db-ea4d.drv
	@./mkppddep.sh redhat /usr/share/cups/model "$@"

-include .debian.$(ARCH).deps
-include .redhat.$(ARCH).deps

.PHONY: security-unlock
security-unlock:
	@if [ -n "$${SSH_CONNECTION}" ] && [ "$$(hostname)" = "dominica" ] && [ "$${USER}" = "macdev" ]; then \
		echo "This appears to be an integration build, attempting to unlock the keychain for signing..."; \
		security unlock -p $$(< ~macdev/.macdev_credentials ) ~/Library/Keychains/tap.keychain-db; \
	fi

##########
# DEBIAN #
##########

DEBIAN_FILTER_TARGETS=debian.$(ARCH)/usr/lib/cups/filter/tecfilter
ifeq ($(DRIVERTYPE),full)
	DEBIAN_FILTER_TARGETS += debian.$(ARCH)/usr/lib/cups/filter/pdftoxps \
	                         debian.$(ARCH)/usr/lib/cups/filter/xpstotpcl \
	                         debian.$(ARCH)/usr/share/cups/mime/toshiba-tec.convs \
	                         debian.$(ARCH)/usr/share/cups/mime/toshiba-tec.types
endif

$(DEBIAN_PKG): $(BUILDTOKEN) $(DEBIAN_FILTER_TARGETS) \
	debian.$(ARCH)/etc/udev/rules.d/69-toshiba-tec-barcode-printers.rules \
    debian.$(ARCH)/usr/bin/tectool \
    debian.$(ARCH)/usr/lib/cups/backend/tec \
	debian.$(ARCH)/usr/lib/toshiba-tec/barcode-printer-drivers/queuemanager \
	debian.$(ARCH)/usr/share/cups/usb/com.toshiba-tec.usb-quirks \
    debian.$(ARCH)/usr/share/doc/toshiba-tec-barcode-printer-drivers/copyright \
    debian.$(ARCH)/usr/share/doc/toshiba-tec-barcode-printer-drivers/license \
    debian.$(ARCH)/usr/share/doc/toshiba-tec-barcode-printer-drivers/changelog.gz \
    debian.$(ARCH)/usr/share/man/man1/tectool.1.gz
	find debian.$(ARCH) -type d -exec chmod 755 {} \;
	fpm -f -a $(DEBARCH) -s dir -t deb -C debian.$(ARCH) -n toshiba-tec-barcode-printer-drivers \
		--before-install support/deb/preinst \
		--after-install support/deb/postinst \
		--before-remove support/deb/prerm \
		--after-remove support/deb/postrm \
		--license 'Proprietary' \
		--maintainer 'TOSHIBA TEC Corporation <support@toshiba-tec.com>' \
		-d 'cups >= 1.3.7' \
		-d 'libc6 >= 2.3' \
		-d 'libusb-0.1-4 >= 0.1.5' \
		-d 'libxml2 >= 2.6.26' \
		-d 'libjpeg62-turbo | libjpeg62' \
		--deb-pre-depends 'debconf >= 0.2.26' \
		--deb-templates support/deb/templates \
		--category 'misc' \
		--description "Drivers for TOSHIBA TEC Barcode Printers\nThis package provides support for TOSHIBA TEC barcode printers" \
		--version "$(VERSION)" \
		--vendor "TOSHIBA TEC CORPORATION" \
		-p "$@"

debian.$(ARCH)/etc/udev/rules.d/69-toshiba-tec-barcode-printers.rules: ../drv/usb-support-files/striped-horse.rules
	$(INSTALL) -D -m 644 "$<" "$@"

debian.$(ARCH)/usr/bin/tectool: ../tectool/build.linux.$(ARCH)/tectool-static
debian.$(ARCH)/usr/lib/cups/backend/tec: ../tecbackend/build.linux.$(ARCH)/tec
debian.$(ARCH)/usr/lib/cups/filter/pdftoxps: ../pdftoxps/build.linux.$(ARCH)/pdftoxps-static
debian.$(ARCH)/usr/lib/cups/filter/xpstotpcl: ../xpstotpcl/build.linux.$(ARCH)/xpstotpcl-static
debian.$(ARCH)/usr/lib/cups/filter/tecfilter: ../tecfilter/build.linux.$(ARCH)/tecfilter-static
debian.$(ARCH)/usr/lib/toshiba-tec/barcode-printer-drivers/queuemanager: ../tecbackend/build.linux.$(ARCH)/queuemanager

debian.$(ARCH)/usr/bin/tectool debian.$(ARCH)/usr/lib/cups/backend/tec debian.$(ARCH)/usr/lib/cups/filter/tecfilter debian.$(ARCH)/usr/lib/cups/filter/xpstotpcl debian.$(ARCH)/usr/lib/cups/filter/pdftoxps debian.$(ARCH)/usr/lib/toshiba-tec/barcode-printer-drivers/queuemanager:
	$(INSTALL) -D -m 755 "$<" "$@"
	$(STRIP) "$@"

debian.$(ARCH)/usr/share/cups/mime/toshiba-tec.%: ../drv/striped-horse.%
	$(INSTALL) -D -m 644 "$<" "$@"

debian.$(ARCH)/usr/share/cups/usb/com.toshiba-tec.usb-quirks: ../drv/usb-support-files/striped-horse.usb-quirks
	$(INSTALL) -D -m 644 "$<" "$@"

debian.$(ARCH)/usr/share/cups/model/%: ../drv/ppd/%
	@mkdir -m 755 -p $$(dirname "$@")
	grep -v "TECTarget" < "$<" > "$@"
	@chmod 644 "$@"

debian.$(ARCH)/usr/share/doc/toshiba-tec-barcode-printer-drivers/%: doc/%
	$(INSTALL) -D -m 644 "$<" "$@"

debian.$(ARCH)/usr/share/doc/toshiba-tec-barcode-printer-drivers/changelog.gz: doc/changelog
	mkdir -m 755 -p $$(dirname "$@")
	gzip -n -9 -c "$<" > "$@"
	chmod 644 "$@"

debian.$(ARCH)/usr/share/man/man1/tectool.1.gz: ../tectool/build.linux.$(ARCH)/tectool.1.gz
	$(INSTALL) -D -m 644 "$<" "$@"

debian.$(ARCH)/DEBIAN/%: support/deb/%
	$(INSTALL) -D -m 755 "$<" "$@"

debian.$(ARCH)/DEBIAN/control: support/deb/control
	mkdir -p debian.$(ARCH)/DEBIAN
	../tools/template.py "$<" "$@" "ARCH=$DEBARCH" "VERSION=${VERSION}"

debian.$(ARCH)/DEBIAN/templates: support/deb/templates
	$(INSTALL) -D -m 644 "$<" "$@"

##########
# REDHAT #
##########

REDHAT_FILTER_TARGETS=redhat.$(ARCH)/usr/lib/cups/filter/tecfilter
ifeq ($(DRIVERTYPE),full)
	REDHAT_FILTER_TARGETS += redhat.$(ARCH)/usr/lib/cups/filter/pdftoxps \
	                         redhat.$(ARCH)/usr/lib/cups/filter/xpstotpcl \
	                         redhat.$(ARCH)/usr/share/cups/mime/toshiba-tec.convs \
	                         redhat.$(ARCH)/usr/share/cups/mime/toshiba-tec.types
endif

$(REDHAT_PKG): $(BUILDTOKEN) $(REDHAT_FILTER_TARGETS) \
	redhat.$(ARCH)/etc/udev/rules.d/69-toshiba-tec-barcode-printers.rules \
    redhat.$(ARCH)/usr/bin/tectool \
    redhat.$(ARCH)/usr/lib/cups/backend/tec \
	redhat.$(ARCH)/usr/lib/toshiba-tec/barcode-printer-drivers/queuemanager \
	redhat.$(ARCH)/usr/share/cups/usb/com.toshiba-tec.usb-quirks \
    redhat.$(ARCH)/usr/share/doc/toshiba-tec-barcode-printer-drivers/copyright \
    redhat.$(ARCH)/usr/share/doc/toshiba-tec-barcode-printer-drivers/license \
    redhat.$(ARCH)/usr/share/man/man1/tectool.1.gz \
    support/rpm/prerm
	chmod -R -s redhat.$(ARCH)
	fpm -f -a $(RPMARCH) -s dir -t rpm -C redhat.$(ARCH) -n toshiba-tec-barcode-printer-drivers \
		-d "cups >= 1.3.7" \
		-d "glibc >= 2.3" \
		-d "libusb >= 0.1.5" \
		-d "libxml2 >= 2.6.25" \
		-d "libjpeg" \
		--description "This is a driver for TOSHIBA TEC barcode printers" \
		--maintainer "support@toshiba-tec.com" \
		--version "$(VERSION)" \
		--vendor "TOSHIBA TEC CORPORATION" \
		--after-install support/rpm/postinst \
		--before-remove support/rpm/prerm \
		-p "$@"

redhat.$(ARCH)/etc/udev/rules.d/69-toshiba-tec-barcode-printers.rules: ../drv/usb-support-files/striped-horse.rules
	$(INSTALL) -D -m 644 "$<" "$@"

redhat.$(ARCH)/usr/bin/tectool: ../tectool/build.linux.$(ARCH)/tectool-static
redhat.$(ARCH)/usr/lib/cups/backend/tec: ../tecbackend/build.linux.$(ARCH)/tec
redhat.$(ARCH)/usr/lib/cups/filter/pdftoxps: ../pdftoxps/build.linux.$(ARCH)/pdftoxps-static
redhat.$(ARCH)/usr/lib/cups/filter/xpstotpcl: ../xpstotpcl/build.linux.$(ARCH)/xpstotpcl-static
redhat.$(ARCH)/usr/lib/cups/filter/tecfilter: ../tecfilter/build.linux.$(ARCH)/tecfilter-static
redhat.$(ARCH)/usr/lib/toshiba-tec/barcode-printer-drivers/queuemanager: ../tecbackend/build.linux.$(ARCH)/queuemanager

redhat.$(ARCH)/usr/bin/tectool redhat.$(ARCH)/usr/lib/cups/backend/tec redhat.$(ARCH)/usr/lib/cups/filter/pdftoxps redhat.$(ARCH)/usr/lib/cups/filter/xpstotpcl redhat.$(ARCH)/usr/lib/cups/filter/tecfilter redhat.$(ARCH)/usr/lib/toshiba-tec/barcode-printer-drivers/queuemanager:
	$(INSTALL) -D -m 755 "$<" "$@"
	$(STRIP) "$@"

redhat.$(ARCH)/usr/share/cups/mime/toshiba-tec.%: ../drv/striped-horse.%
	$(INSTALL) -D -m 644 "$<" "$@"

redhat.$(ARCH)/usr/share/cups/usb/com.toshiba-tec.usb-quirks: ../drv/usb-support-files/striped-horse.usb-quirks
	$(INSTALL) -D -m 644 "$<" "$@"

redhat.$(ARCH)/usr/share/cups/model/%: ../drv/ppd/%
	@mkdir -p $$(dirname "$@")
	grep -v "TECTarget" < "$<" > "$@"
	@chmod 644 "$@"

redhat.$(ARCH)/usr/share/doc/toshiba-tec-barcode-printer-drivers/%: doc/%
	$(INSTALL) -D -m 644 "$<" "$@"

redhat.$(ARCH)/usr/share/man/man1/tectool.1.gz: ../tectool/build.linux.$(ARCH)/tectool.1.gz
	$(INSTALL) -D -m 644 "$<" "$@"

.PHONY: clean
clean:
	rm -rf debian.$(ARCH)
	rm -rf redhat.$(ARCH)
	-rm -f .redhat.$(ARCH).deps
	-rm -f .debian.$(ARCH).deps
	rm -rf build

$(BUILDTOKEN):
	mkdir -p $(shell dirname "$(BUILDTOKEN)")
	date | cksum > "$@"
