#ifndef STRIPEDHORSE_DEVICE_H
#define STRIPEDHORSE_DEVICE_H

#include <cstdint>
#include <memory>
#include <tuple>
#include <vector>
#include <libcore/exceptions.h>
#include <libcore/uri.h>

namespace StripedHorse
{
	class Device
	{
	public:
		static bool PerformDiscovery(std::vector<Device> &devices);
		
		Device(const URI &uri);
		Device(const Device &) = delete;
		Device(Device &&);
		~Device();

		const std::string &getDescription() const;
		const std::string &getDeviceID() const;
		const std::string &getDeviceInfo() const;
		const URI &getURI() const;
		std::tuple<size_t, size_t> sendData(std::istream &in);

	private:
		URI m_uri;
		std::string m_description;
		std::string m_deviceID;
		std::string m_deviceInfo;
	};

	class DeviceNotFoundException : public Exception
	{
	public:
		DeviceNotFoundException(const std::string &msg) : Exception(msg) {}
	};
}

#endif
