#include <libcore/logging.h>
#include "cupssync.h"

StripedHorse::CUPSSync::CUPSSync(std::istream &in)
{
	char syncWordBytes[4];
	if (!in.read(reinterpret_cast<char *>(&syncWordBytes[0]), sizeof syncWordBytes))
		throw IOException("Unable to read from input");
	
	std::string syncWord(syncWordBytes, sizeof syncWordBytes);
	
	if (syncWord == "RaS3")
	{
		m_version = 3;
		m_endianness = kBigEndian;
	}
	else if (syncWord == "RaS2")
	{
		m_version = 2;
		m_endianness = kBigEndian;
	}
	else if (syncWord == "RaSt")
	{
		m_version = 1;
		m_endianness = kBigEndian;
	}
	else if (syncWord == "3SaR")
	{
		m_version = 3;
		m_endianness = kLittleEndian;
	}
	else if (syncWord == "2SaR")
	{
		m_version = 2;
		m_endianness = kLittleEndian;
	}
	else if (syncWord == "tSaR")
	{
		m_version = 1;
		m_endianness = kLittleEndian;
	}
	else
	{
		throw CUPSSyncException("Format not understood");
	}
	
	Logging::Debug() << "Format endianness: " << (m_endianness == kBigEndian ? "big" : "little") << std::endl;
}

enum StripedHorse::Endian StripedHorse::CUPSSync::getFormatEndianness() const
{
	return m_endianness;
}

int StripedHorse::CUPSSync::getVersion() const
{
	return m_version;
}
