#ifndef STRIPEDHORSE_DBEA4DFILTER_H
#define STRIPEDHORSE_DBEA4DFILTER_H

#include "filter.h"

namespace StripedHorse
{
	class DBEA4DFilter : public Filter
	{
	public:
		DBEA4DFilter(std::istream &i, std::ostream &o, const Options &op);
		~DBEA4DFilter();
	
	protected:
		void beginDocument() override;
		void endDocument() override;
		void processPageBitmap(const LemonGFX::Bitmap &bitmap, const CUPSFormat &format) override;
		bool shouldTumbleBitmap() const override;
		virtual bool tryGetStatusCode(int &code) const override;

	private:
		enum class PrintingSide
		{
			kFront,
			kBack
		};

		enum class CuttingMode
		{
			kNoCutting,
			kCutAfterEveryPage,
			kCutAfterDocument
		};

		// this is a driver setting
		enum class TwoSidedPrintingMode
		{
			kSimplex,
			kDuplex,
			kDuplexTumble
		};

		// this is a printer setting
		enum class ThermalPrintingMode
		{
			kSimplexFront,
			kSimplexBack,
			kDuplex
		};

		PrintingSide determinePrintingSide() const;
		void feedRows(size_t rows);
		void initialisePrinter();
		void performKnifeCut();
		void print();
		void setPageHeight(size_t height);
		void setPrintingSide(PrintingSide);
		void setThermalPrintingMode(ThermalPrintingMode);

		TwoSidedPrintingMode m_twoSidedPrintingMode;
		bool m_printOnBackOnly;
		CuttingMode m_cuttingMode;
		uint32_t m_lastPageHeight;
	};
}

#endif
