#ifndef STRIPEDHORSE_INSERTION_H
#define STRIPEDHORSE_INSERTION_H

#include <string>

#include <libcore/types.h>

namespace StripedHorse
{
	class Insertion
	{
	public:
		enum InsertionType
		{
			None,
			String,
			File
		};
		
		/**
		 Constructs an insertion that does nothing
		 */
		Insertion();
		
		/**
		 Copies an insertion from another insertion
		 */
		Insertion(const Insertion &);
		
		/**
		 Moves an insertion from another insertion
		 */
		Insertion(Insertion &&);
		
		/**
		 Constructs an insertion object
		 @param[in] insertionType the type of insertion
		 @param[in] tmpl the template of the insertion
		 */
		Insertion(InsertionType insertionType, const std::string &tmpl);
		
		/**
		 Copies an insertion object from another insertion object
		 */
		Insertion &operator=(const Insertion &other);
		
		/**
		 Gets the type of this insertion
		 @return either None, File, or Strin
		 */
		enum InsertionType getInsertionType() const;
		
		/**
		 Returns the insertion after all template parameters have been replaced
		 @param[in] dict the dictionary containing the definitions of the
		            template parameters, e.g. the template paramter '%t' might
		            be the current time, etc.
		 @return if this insertion type is None, then an empty string is
		         returned. If the insertion type is String, then the template
		         argument provided during construction has its template
		         parameters replaced with the definitions in the dictionary
		         argument, and the result is returned. If the insertion type is
		         File, then the same process is done as if it were String, but
		         the result is instead used as a filename. If the file exists,
		         its contents are read and if it contains any template
		         parameters then these are also replaced and the result is
		         returned. If the file does not exist, an empty string is
		         returned.
		 */
		std::string getInsertion(const Dictionary &dict) const;
		
		/**
		 Gets the template provided during construction without any replacements
		 being made.
		 */
		std::string getTemplate() const;
		
	private:
		enum InsertionType m_insertionType = None;
		std::string m_template;
	};
}

#endif

