#include <climits>
#include <regex>
#include "cmd-setfineadjustments.h"

namespace
{
	class FixedPointException : public StripedHorse::Exception
	{
	public:
		FixedPointException() : Exception("Invalid floating point") {}
	};
	
	int readFixedPoint(const std::string &input)
	{
		static std::regex fixedPointRegex("^(-|\\+)?(\\d{1,5})(\\.(\\d))?$");
		
		std::smatch result;
		if (!std::regex_match(input, result, fixedPointRegex))
			throw FixedPointException();
		
		// double checking
		if (static_cast<std::string::size_type>(result.length()) != input.length())
			throw FixedPointException();
		
		int integerPart = std::stoi(result[2]);
		int fractionalPart = 0;
		
		if (result.length(4))
			fractionalPart = std::stoi(result[4]);
		
		int fullInteger = integerPart * 10 + fractionalPart;
		
		if (result.length(1) == 1 && result[1] == "-")
			fullInteger = -fullInteger;
		
		return fullInteger;
	}
}

void StripedHorse::TECTool::Commands::SetFineAdjustments(Printer &printer, const ArgumentList &args)
{
	if (args.size() != 3)
		throw InvalidNumberOfArgumentsException(args.size(), 3, 0);
	
	int feedAdjustment = 0;
	int cutOrStripAdjustment = 0;
	int backfeedAdjustment = 0;
	
	try
	{
		feedAdjustment = readFixedPoint(args[0]);
		if (feedAdjustment < -500 || feedAdjustment > 500)
			throw ArgumentOutOfRangeException("feed", feedAdjustment / 10.0, -50.0, 50.0);
	}
	catch (FixedPointException &)
	{
		throw InvalidArgumentException("feed", args[0]);
	}
	
	try
	{
		cutOrStripAdjustment = readFixedPoint(args[1]);
		if (cutOrStripAdjustment < -500 || cutOrStripAdjustment > 500)
			throw ArgumentOutOfRangeException("cutstrip", cutOrStripAdjustment / 10.0, -50.0, 50.0);
	}
	catch (FixedPointException &)
	{
		throw InvalidArgumentException("cutstrip", args[1]);
	}
	
	try
	{
		backfeedAdjustment = readFixedPoint(args[2]);
		if (backfeedAdjustment < -99 || backfeedAdjustment > 99)
			throw ArgumentOutOfRangeException("backfeed", backfeedAdjustment / 10.0, -9.9, 9.9);
	}
	catch (FixedPointException &)
	{
		throw InvalidArgumentException("backfeed", args[2]);
	}
	
	printer.writeCommand("AX;+bbb,+ddd,+ff", feedAdjustment, cutOrStripAdjustment, backfeedAdjustment);
}
