#include <sstream>
#include "tectool-command.h"

namespace
{
	std::string generateInvalidNumberOfArgumentsMessage(int given, int expected, int optional)
	{
		std::ostringstream ss;
		ss << "Unexpected number of arguments given (" << given << "). Expected " << expected << " arguments";
		if (optional > 0)
			ss << " (with " << optional << " optional arguments)";
		return ss.str();
	}
	
	std::string generateInvalidArgumentMessage(const std::string &argName, const std::string &argValue)
	{
		std::ostringstream ss;
		ss << "Invalid value for argument \"" << argName << "\": " << argValue;
		return ss.str();
	}
}

StripedHorse::TECTool::InvalidNumberOfArgumentsException::InvalidNumberOfArgumentsException(int given, int expected, int optional)
  : Exception(generateInvalidNumberOfArgumentsMessage(given, expected, optional))
{
}

StripedHorse::TECTool::InvalidArgumentException::InvalidArgumentException(const std::string &argName, const std::string &argValue)
  : Exception(generateInvalidArgumentMessage(argName, argValue))
{
}

StripedHorse::TECTool::InvalidArgumentException::InvalidArgumentException(const std::string &msg)
  : Exception(msg)
{
}

StripedHorse::TECTool::ResponseException::ResponseException(const std::string &msg)
  : Exception(msg)
{
}

StripedHorse::TECTool::InvalidResponseException::InvalidResponseException(const std::string &msg)
  : ResponseException(msg)
{
}
