#ifndef STRIPEDHORSE_TECTOOL_COMMAND_H
#define STRIPEDHORSE_TECTOOL_COMMAND_H

#include <functional>
#include <string>
#include <vector>

#include <libcore/exceptions.h>

#include "printer.h"

namespace StripedHorse
{
	namespace TECTool
	{
		typedef std::vector<std::string> ArgumentList;
		
		typedef std::function<void(Printer &, const ArgumentList &)> Command;
		
		class InvalidNumberOfArgumentsException : public Exception
		{
		public:
			InvalidNumberOfArgumentsException(int provided, int expected, int optional = 0);
		};
		
		class InvalidArgumentException : public Exception
		{
		public:
			InvalidArgumentException(const std::string &argName, const std::string &argValue);
			InvalidArgumentException(const std::string &msg);
		};
		
		class ArgumentOutOfRangeException : public Exception
		{
		public:
			template<typename T>
			ArgumentOutOfRangeException(const std::string &name, const T &val, const T &min, const T &max) : Exception("Argument \"" + name + "\" out of range (" + std::to_string(val) + "). Min = " + std::to_string(min) + ", max = " + std::to_string(max)) {}
		};
		
		class ResponseException : public Exception
		{
		public:
			ResponseException(const std::string &msg);
		};
		
		class InvalidResponseException : public ResponseException
		{
		public:
			InvalidResponseException(const std::string &msg);
		};
	}
}

#endif
