# This Makefile is designed to be included into another Makefile for the purpose of generating
# static and dynamic libraries. The Makefile that includes this file should declare at least
# two variables, SOURCES and EXEC_NAME, containing the source files and name of the library
# respectively. It may also choose to override CFLAGS and LDFLAGS etc.

OBJECTS=$(addprefix $(BUILDDIR)/, $(SOURCES:.cpp=.obj)) 
EXEC=$(BUILDDIR)/$(EXEC_NAME).exe
STATIC_EXEC=$(BUILDDIR)/$(EXEC_NAME)-static.exe

ARCH?=i386
CXX=cl
LD=link
CP=copy /Y
RM=del /F /S /Q
BUILDDIR=build.win32.$(ARCH)

override CXXFLAGS+=/std:c++17 /nologo /c /EHsc /Z7 /MT /W4 /wd4244 /D _CRT_SECURE_NO_WARNINGS=1 /D _USE_MATH_DEFINES=1 /I.. /I..\3rdparty\include
override LDFLAGS+=/debug /libpath:..\3rdparty\lib $(WIN32_LDFLAGS)

# Handle all dependencies
ifneq ($(DEPENDS),)
LIB_DEPENDS=$(patsubst %,%.lib,$(patsubst lib%,%,$(notdir $(DEPENDS))))
STATIC_LIB_DEPENDS=$(addsuffix .lib,$(join $(addsuffix /$(BUILDDIR)/,$(DEPENDS)),$(patsubst lib%,%,$(notdir $(DEPENDS)))))
DYNAMIC_LDFLAGS=$(addprefix /libpath:,$(addsuffix /$(BUILDDIR),$(DEPENDS)))
endif

.PHONY: all
all: $(BUILDDIR) $(EXEC) $(STATIC_EXEC)

$(BUILDDIR):
	mkdir $@

.PHONY: install
install:
	@if [$(INSTALL_DIR)] == [] (echo "Must set INSTALL_DIR" && exit 1)
	$(CP) "$(subst /,\,$(STATIC_EXEC))" "$(subst /,\,$(INSTALL_DIR))"
	$(CP) "$(subst /,\,$(EXEC))" "$(subst /,\,$(INSTALL_DIR))"

.PHONY: clean
clean:
	-$(RM) $(BUILDDIR)

$(EXEC): $(OBJECTS)
	$(LD) /pdb:$(BUILDDIR)\\$(EXEC_NAME).pdb $^ $(LDFLAGS) $(DYNAMIC_LDFLAGS) $(LIB_DEPENDS) /OUT:$@

$(STATIC_EXEC): $(OBJECTS) $(STATIC_LIB_DEPENDS)
	$(LD) /pdb:$(BUILDDIR)/$(EXEC_NAME)-static.pdb $^ $(LDFLAGS) /OUT:$@

$(BUILDDIR)/%.obj: %.cpp
	$(CXX) $(CFLAGS) $(WIN32_CFLAGS) $(CXXFLAGS) /Fo$@ $<
