# This is the same thing as library.mk but for Windows builds

ARCH?=i386
CXX=cl
AR=lib
LD=link
CP=copy /Y
RM=del /F /S
BUILDDIR=build.win32.$(ARCH)

LIB_OBJECTS=$(addprefix $(BUILDDIR)/, $(LIB_SOURCES:.cpp=.obj)) 

# Default flags
override CXXFLAGS+=/std:c++17 /nologo /c /EHsc /Z7 /MT /W4 /wd4244 /D _CRT_SECURE_NO_WARNINGS=1 /D _USE_MATH_DEFINES=1 /I.. /I..\\3rdparty\\include
override LDFLAGS+=/nologo /debug /dll /libpath:..\\3rdparty\\lib /pdb:$(BUILDDIR)\\$(LIB_NAME).pdb $(WIN32_LDFLAGS)
override ARFLAGS=/nologo

# Handle all dependencies
ifneq ($(DEPENDS),)
LIB_DEPENDS=$(patsubst %,%.lib,$(patsubst lib%,%,$(notdir $(DEPENDS))))
override LDFLAGS+=$(addprefix /libpath:,$(addsuffix /$(BUILDDIR),$(DEPENDS)))
endif

SHARED_LIB_EXEC=$(BUILDDIR)/$(LIB_NAME).dll
STATIC_LIB_EXEC=$(BUILDDIR)/$(LIB_NAME).lib

.PHONY: all
all: $(BUILDDIR) $(SHARED_LIB_EXEC) $(STATIC_LIB_EXEC)

$(BUILDDIR):
	-mkdir $@	

$(LIB_OBJECTS): | $(BUILDDIR)

$(STATIC_LIB_EXEC): $(LIB_OBJECTS)
	$(AR) $(ARFLAGS) /out:$@ $^

$(SHARED_LIB_EXEC): $(LIB_OBJECTS)
	$(LD) $(LDFLAGS) /out:$(SHARED_LIB_EXEC) $(LIB_DEPENDS) $^

$(BUILDDIR)/%.obj: %.cpp
	$(CXX) $(CFLAGS) $(WIN32_CFLAGS) $(CXXFLAGS) /Fo$@ $<

install: $(SHARED_LIB_EXEC) $(STATIC_LIB_EXEC)
	@if [$(INSTALL_DIR)] == [] (echo "Must set INSTALL_DIR" && exit 1)
	$(CP) "$(subst /,\\,$(STATIC_LIB_EXEC))" "$(subst /,\\,$(INSTALL_DIR))"
	$(CP) "$(subst /,\\,$(SHARED_LIB_EXEC))" "$(subst /,\\,$(INSTALL_DIR))"

.PHONY: clean
clean:
	-$(RM) $(subst /,\\,$(BUILDDIR))
