#!/usr/bin/env python

# workaround for no ZIP file support on Windows

import optparse
import os
import sys
import zipfile


def add_file_to_zip(z, opts, filename):
    if not opts.quiet:
        sys.stderr.write("adding: %s\n" % filename)
    
    if opts.junk_directory_name:
        arcname = os.path.basename(filename)
    else:
        arcname = filename.replace('\\', '/')
    
    if os.path.islink(filename):
        zinfo = zipfile.ZipInfo(arcname)
        zinfo.create_system = 3
        zinfo.external_attr = 0xA1ED0000
        z.writestr(zinfo, os.readlink(filename))
    else:
        z.write(filename, arcname)


def walk_handler(arg, dirname, names):
    for infile in names:
        fullpath = os.path.join(dirname, infile)
        if os.path.isdir(infile):
            os.path.walk(fullpath, walk_handler, arg)
        else:
            add_file_to_zip(arg["zipfile"], arg["options"], fullpath)


parser = optparse.OptionParser(usage="usage: %prog [ options ] zip-file in-file [ in-file ... ]")
parser.add_option("-q", dest="quiet", help="do not output filenames during zipping", action="store_true")
parser.add_option("-r", dest="recursive", help="recursively include files", action="store_true")
parser.add_option("-y", dest="preserve_links", help="preserve links", action="store_true")
parser.add_option("-j", dest="junk_directory_name", help="junk (don't record) directory names", action="store_true")
parser.add_option("-X", dest="exclude_attributes", help="exclude attributes", action="store_true")
(opts, args) = parser.parse_args(sys.argv)

if len(args) < 3:
    parser.error("need output zip filename and at least one file to place in archive")

z = zipfile.ZipFile(args[1], "w", zipfile.ZIP_DEFLATED)

for infile in args[2:]:
    if os.path.isdir(infile):
        if opts.recursive:
            os.path.walk(infile, walk_handler, { "zipfile" : z, "options" : opts })
        else:
            sys.stderr.write("skipping directory %s\n" % infile)
    else:
        add_file_to_zip(z, opts, infile)

z.close()
