#include <cctype>
#include <regex>
#include <sstream>
#include "font-utilities.h"

namespace
{
	// strips any bold/italic suffix, strips spaces, and makes it lowercase
	std::string reduceFontName(const std::string &fontName)
	{
		std::regex suffixRegex { "[ -]?(([Bb]old[ -]*[Ii]talic)|[Bb]old|[Ii]talic)$" };
		std::smatch matchResult;
		std::string newFontName;

		newFontName.reserve(fontName.size());

		if (std::regex_match(fontName, matchResult, suffixRegex))
			newFontName.append(fontName.substr(0, matchResult.position(0)));
		
		for (const auto &ch : fontName)
			if (!::isspace(ch) && ch != '-')
				newFontName.push_back(::tolower(ch));
		
		return newFontName;
	}
}

int StripedHorse::FontUtilities::CompareFontNames(const std::string &font1, const std::string &font2)
{
	return reduceFontName(font1).compare(reduceFontName(font2));
}

std::string StripedHorse::FontUtilities::GetBaseName(const std::string &fontName)
{
	std::regex fontNameRegex { "^([A-Z]{6}\\+)?(.+)$" };
	std::smatch result;
	if (std::regex_match(fontName, result, fontNameRegex))
	{
		return result.str(2);
	}
	else
	{
		return fontName;
	}
}

std::tuple<std::string, std::string, std::string> StripedHorse::FontUtilities::ParseFontDescription(const std::string &fontDescription)
{
	std::string substitute, genericName, specificName, flags;
	std::istringstream ss(fontDescription);
	std::getline(ss, substitute, ':');
	std::getline(ss, genericName, ':');
	std::getline(ss, specificName, ':');
	std::getline(ss, flags);
	if ((ss.bad() && !ss.eof()) || substitute != "#substitute")
		throw StripedHorse::InvalidArgumentException("Invalid font description");
	
	return std::make_tuple(genericName, GetBaseName(specificName), flags);
}

std::tuple<bool, bool> StripedHorse::FontUtilities::ParseFontDescriptionFlags(const std::string &flags)
{
	bool isBold = flags.find("bold") != std::string::npos;
	bool isItalic = flags.find("italic") != std::string::npos;

	return std::make_tuple(isBold, isItalic);
}
