#ifndef STRIPEDHORSE_FONT_UTILITIES_H
#define STRIPEDHORSE_FONT_UTILITIES_H

#include <algorithm>
#include <unordered_map>
#include <string>
#include <libcore/exceptions.h>

namespace StripedHorse
{
	namespace FontUtilities
	{
		using FontNameMap = std::unordered_map<std::string, std::string>;

		int CompareFontNames(const std::string &font1, const std::string &font2);
		std::string GetBaseName(const std::string &fontName);
		std::tuple<std::string, std::string, std::string> ParseFontDescription(const std::string &fontDescription);
		std::tuple<bool, bool> ParseFontDescriptionFlags(const std::string &fontDescriptionFlags);

		template<typename T>
		std::string MapFontName(const std::string &name, const FontNameMap &specificFontMap, const std::string &generic, const FontNameMap &genericFontMap, const std::vector<T> &fontTable, bool allowSubstitution)
		{
			std::string result;

			if (allowSubstitution)
			{
				auto specificSubbedNameIt = specificFontMap.find(name);
				if (specificSubbedNameIt != specificFontMap.end())
					return specificSubbedNameIt->second;
			}

			// If no font substitution has occurred, look for exact match in printer's font table
			auto inTable = std::find_if(fontTable.begin(), fontTable.end(), [&name](const T &f){
				return CompareFontNames(f.name, name) == 0;
			});
		
			if (inTable != fontTable.end())
				return inTable->name;
			
			// If no font substitution has occurred, use generic font name
			if (allowSubstitution)
			{
				auto mappedNameIt = genericFontMap.find(generic);
				if (mappedNameIt != genericFontMap.end())
					return mappedNameIt->second;
			}
			
			throw StripedHorse::UnimplementedException("Cannot substitute this font");
		}
	}
}

#endif
