#include <libcore/exceptions.h>
#include <libtecfilter/commands.h>
#include "templateobject.h"

StripedHorse::TemplateObject::TemplateObject(const RectangleF &region, const std::string &data)
  : m_region(region), m_data(data), m_rotation(Rotation::kRotate0)
{
}

StripedHorse::TemplateObject::~TemplateObject()
{
}

std::string StripedHorse::TemplateObject::getData() const
{
	return m_data;
}

StripedHorse::PointF StripedHorse::TemplateObject::getPlacementPosition(RectangleF::type innerWidth, RectangleF::type innerHeight) const
{
	RectangleF::type regionWidth = m_region.getWidth();
	RectangleF::type regionHeight = m_region.getHeight();
	PointF result;

	switch (m_rotation)
	{
		case Rotation::kRotate0:
			result.x = m_region.lowerLeft.x + (regionWidth - innerWidth) / 2.0;
			result.y = m_region.lowerLeft.y - (regionHeight - innerHeight) / 2.0;
			break;
			
		case Rotation::kRotate90:
			result.x = m_region.upperRight.x - (regionWidth - innerHeight) / 2.0;
			result.y = m_region.lowerLeft.y + (regionHeight - innerWidth) / 2.0;
			break;

		case Rotation::kRotate180:
			result.x = m_region.upperRight.x - (regionWidth - innerWidth) / 2.0;
			result.y = m_region.upperRight.y + (regionHeight - innerHeight) / 2.0;
			break;

		case Rotation::kRotate270:
			result.x = m_region.lowerLeft.x + (regionWidth - innerHeight) / 2.0;
			result.y = m_region.upperRight.y - (regionHeight - innerWidth) / 2.0;
			break;
			
		default:
			throw InternalInconsistencyException("Invalid rotation of template object");
	}

	return result;
}

StripedHorse::RectangleF StripedHorse::TemplateObject::getRegion() const
{
	return m_region;
}

StripedHorse::Rotation StripedHorse::TemplateObject::getRotation() const
{
	return m_rotation;
}

uint32_t StripedHorse::TemplateObject::getTPCLRotation() const
{
	switch (m_rotation)
	{
		case Rotation::kRotate0:
			return 0;
		
		case Rotation::kRotate90:
			return 1;
		
		case Rotation::kRotate180:
			return 2;

		case Rotation::kRotate270:
			return 3;
		
		case Rotation::kRotateOther:
		default:
			throw InternalInconsistencyException("Invalid rotation of template object");
	}
}

void StripedHorse::TemplateObject::setRotation(enum Rotation r)
{
	m_rotation = r;
}

void StripedHorse::TemplateObject::write(TPCLWriter &writer, double dpmm) const
{
	// not used in default implementation of write()
	(void) dpmm;

	writer.writeCommandStart("AAA;");
	writer.writeString("UNUSED TEMPLATE OBJECT");
	writer.writeCommandEnd();
}
