#include <cmath>
#include <libtecfilter/commands.h>
#include "tobj-code93.h"

StripedHorse::Code93TemplateObject::Code93TemplateObject(const RectangleF &region, const std::string &data)
  : TemplateObject(region, data)
{
}

void StripedHorse::Code93TemplateObject::write(TPCLWriter &writer, double dpmm) const
{
	std::string data = getData();
	const RectangleF &region = getRegion();

	RectangleF::type height = 0;
	RectangleF::type width = 0;
	switch (getRotation())
	{
		case Rotation::kRotate0:
		case Rotation::kRotate180:
		case Rotation::kRotateOther:
			height = region.getHeight();
			width = region.getWidth();
			break;
		
		case Rotation::kRotate90:
		case Rotation::kRotate270:
			height = region.getWidth();
			width = region.getHeight();
			break;
	}

	
	// 9 bars for every character, plus the start/stop characters, plus the two checksum characters
	// and one more bar for the termination bar.
	RectangleF::type totalBarcodeWidth = ((data.size() + 4) * 9 + 1) * 10.0 / dpmm;
	
	uint32_t zoom = static_cast<uint32_t>(std::abs(width) / totalBarcodeWidth);
	if (zoom == 0)
		zoom = 1;
	else if (zoom > 15)
		zoom = 15;

	PointF pos = getPlacementPosition(totalBarcodeWidth * zoom, height);

	writer.writeCommand("XBaa;bbbb,cccc,d,e,ff,k,llll=s/256", 0U, static_cast<uint32_t>(pos.x), static_cast<uint32_t>(pos.y), 'C', 1, zoom, getTPCLRotation(), static_cast<uint32_t>(height), data);
}
