#ifndef STRIPEDHORSE_TPCLCONTEXT_H
#define STRIPEDHORSE_TPCLCONTEXT_H

#include <functional>
#include <iostream>
#include <string>
#include <unordered_map>
#include <vector>
#include <lemongfx/bitmap.h>
#include <lemongfx/path.h>
#include <libcore/exceptions.h>
#include <libcore/types.h>
#include <libtecfilter/options.h>
#include <libtecfilter/tpclwriter.h>
#include <lemonxps/pageelementcontext.h>
#include "stringapproximator.h"
#include "templateobject.h"

namespace StripedHorse
{
	class TPCLContext : public LemonXPS::IPageElementContext
	{
	public:
		TPCLContext(std::ostream &);
		TPCLContext(std::ostream &, const Options &options);
		~TPCLContext();

		void beginPageElement(LemonXPS::PageElementCPtr) override;
		void endPageElement(LemonXPS::PageElementCPtr) override;
		LemonGFX::Path &getCurrentPath();
		const LemonGFX::Path &getCurrentPath() const;
		TransformMatrix &topMatrix();

	private:
		enum class BitmapCompressionMode : char
		{
			kNone = '1',
			kAuto = '?',
			kTOPIX = '3'
		};

		void beginCanvas(LemonXPS::PageElementCPtr);
		void beginDetectedShape(LemonXPS::PageElementCPtr);
		void beginFixedPage(LemonXPS::PageElementCPtr);
		void beginGlyphs(LemonXPS::PageElementCPtr);
		void beginImageBrush(LemonXPS::PageElementCPtr);
		void beginPath(LemonXPS::PageElementCPtr);
		void beginPath_Data(LemonXPS::PageElementCPtr);
		void beginPath_Fill(LemonXPS::PageElementCPtr);
		void beginPathFigure(LemonXPS::PageElementCPtr);
		void beginPathGeometry(LemonXPS::PageElementCPtr);
		void beginPolyBezierSegment(LemonXPS::PageElementCPtr);
		void beginPolyLineSegment(LemonXPS::PageElementCPtr);
		void endCanvas(LemonXPS::PageElementCPtr);
		void endFixedPage(LemonXPS::PageElementCPtr);
		void endPath(LemonXPS::PageElementCPtr);
		void endPathFigure(LemonXPS::PageElementCPtr);

		void flushPageBitmap();
		void writeBitmap(const LemonGFX::Bitmap &bmp);

		using Handler = void (TPCLContext::*)(LemonXPS::PageElementCPtr);
		using HandlerMap = std::unordered_map<std::string, Handler>;
		using PointU32 = StripedHorse::BasicPoint<uint32_t>;

		TPCLWriter m_writer;

		HandlerMap m_beginHandlers;
		HandlerMap m_endHandlers;
		std::vector<TransformMatrix> m_matrixStack;
		Options m_options;

		// cached options
		enum BitmapCompressionMode m_bitmapCompressionMode;
		double m_dpmm;
		bool m_rasterAllLines;
		
		// template objects
		std::vector<TemplateObjectPtr> m_templateObjects;

		// Graphics state
		std::string m_currentFillRule;
		LemonXPS::PageElementCPtr m_currentPathElement;
		LemonGFX::Path m_currentPath;
		StringApproximator m_stringApproximator;
		std::unique_ptr<LemonGFX::Bitmap> m_imageBrush;
		LemonGFX::Bitmap m_pageBitmap;
	};

	class TPCLContextException : public Exception
	{
	public:
		TPCLContextException(const std::string &msg) : Exception(msg) {}
	};
}

#endif
