#include <iterator>
#include <unordered_map>
#include <vector>
#include "font-utilities.h"
#include "vectorfont.h"

namespace
{
	using FontType = StripedHorse::TPCLVectorFont::Type;
	std::vector<StripedHorse::TPCLVectorFont> g_vectorFontTable {
		{ "Helvetica-fixed",               "bold",   "A",  FontType::kOutline },
		{ "Helvetica-fixed",               "italic", "A",  FontType::kOutline },
		{ "Helvetica-fixed",               "normal", "A",  FontType::kOutline },
		{ "Helvetica",                     "bold",   "B",  FontType::kOutline },
		{ "Helvetica",                     "italic", "B",  FontType::kOutline },
		{ "Helvetica",                     "normal", "B",  FontType::kOutline },
		{ "Times Roman",                   "bold",   "H",  FontType::kOutline },
		{ "Times Roman",                   "italic", "H",  FontType::kOutline },
		{ "Times Roman",                   "normal", "H",  FontType::kOutline },
		{ "BalloonPExtBol",                "normal", "01", FontType::kTrueType },
		{ "BlacklightD",                   "normal", "02", FontType::kTrueType },
		{ "BrushScrD",                     "normal", "03", FontType::kTrueType },
		{ "CG Times",                      "normal", "04", FontType::kTrueType },
		{ "CG Times",                      "bold",   "05", FontType::kTrueType },
		{ "CG Times",                      "italic", "06", FontType::kTrueType },
		{ "Clarendon Condensed",           "bold",   "07", FontType::kTrueType },
		{ "FlashPBol",                     "normal", "08", FontType::kTrueType },
		{ "Garamond Kursiv Halbfett",      "normal", "09", FontType::kTrueType },
		{ "GoudyHeaP",                     "normal", "10", FontType::kTrueType },
		{ "GilliesGotDBol",                "bold",   "11", FontType::kTrueType },
		{ "GilliesGotLig",                 "normal", "12", FontType::kTrueType },
		{ "NimbusSanNovTUltLigCon",        "normal", "13", FontType::kTrueType },
		{ "Ryahd",                         "normal", "14", FontType::kTrueType },
		{ "Ryahd",                         "bold",   "15", FontType::kTrueType },
		{ "CG Triumvirate",                "normal", "16", FontType::kTrueType },
		{ "CG Triumvirate Condensed",      "bold",   "17", FontType::kTrueType },
		{ "Univers Medium",                "normal", "18", FontType::kTrueType },
		{ "Univers",                       "bold",   "19", FontType::kTrueType },
		{ "Univers Medium",                "italic", "20", FontType::kTrueType },
	};

	std::unordered_map<std::string, std::string> g_genericFontMapping {
		{ "fixed-width", "Helvetica-fixed" },
		{ "sans-serif",  "Helvetica"       },
		{ "serif",       "Times Roman"     }
	};

	std::unordered_map<std::string, std::string> g_specificFontMapping {
		{ "Liberation Serif", "Times Roman" }
	};
}

StripedHorse::TPCLVectorFont::TPCLVectorFont()
  : valid(false)
{
}

StripedHorse::TPCLVectorFont::TPCLVectorFont(const std::string &name, const std::string &style, const std::string &code, enum Type t)
  : name(name), style(style), code(code), type(t), valid(true)
{
}

StripedHorse::TPCLVectorFontParameters::TPCLVectorFontParameters()
{
}

StripedHorse::TPCLVectorFontParameters::TPCLVectorFontParameters(double dpmm, double fontSize, const std::string &fontDescription, bool allowSubstitution)
  : characterWidth(0), characterHeight(0)
{
	// dpmm not used for now
	(void) dpmm;

	std::string genericName, specificName, flags;
	bool isBold, isItalic;
	
	std::string finalFontName;

	try
	{
		std::tie(genericName, specificName, flags) = FontUtilities::ParseFontDescription(fontDescription);
		std::tie(isBold, isItalic) = FontUtilities::ParseFontDescriptionFlags(flags);

		finalFontName = FontUtilities::MapFontName(specificName, g_specificFontMapping, genericName, g_genericFontMapping, g_vectorFontTable, allowSubstitution);
	}
	catch (StripedHorse::UnimplementedException &)
	{
		return;
	}	
	
	std::vector<TPCLVectorFont> availableFonts;
	std::copy_if(g_vectorFontTable.begin(), g_vectorFontTable.end(), std::back_inserter(availableFonts), [=](const TPCLVectorFont &bf) {
		return (bf.name == finalFontName) && ((isItalic && bf.style == "italic") || (isBold && bf.style == "bold") || (!isItalic && !isBold && (bf.style == "normal")));
	});

	if (availableFonts.size() == 0)
		throw StripedHorse::InvalidArgumentException("No suitable font to use");
	
	font = availableFonts.front();
	characterWidth = std::max(20U, static_cast<uint32_t>(fontSize));
	characterHeight = characterWidth;
}
